package org.opendof.tools.repository.interfaces.cli.cmds.group;

import java.util.Date;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.opendof.tools.repository.interfaces.cli.util.CliBase;
import org.opendof.tools.repository.interfaces.cli.util.CliCommand;
import org.opendof.tools.repository.interfaces.cli.util.StrH;
import org.opendof.tools.repository.interfaces.da.SubmitterData;

@SuppressWarnings("javadoc")
public class AddGroupCommand extends GroupCommand
{
    // c, h, l, v used
    public static final String DescShortCl = "d";
    public static final String DescLongCl = "description";
    
    public static final String GroupShortCl = "g";
    public static final String GroupLongCl = "group";

    public static final String ManagerShortCl = "a";
    public static final String ManagerLongCl = "group-admin";

    public static final String MemberShortCl = "m";
    public static final String MemberLongCl = "member";

    private volatile AddCmdData addCmdData;
    
    public AddGroupCommand(CliBase cliBase, CliCommand previousCommand, String commandName)
    {
        super(cliBase, previousCommand, commandName);
    }

    public AddCmdData getAddCmdData()
    {
        return addCmdData;
    }
    
    @Override
    protected void cliSetup()
    {
        super.cliSetup();
        //@formatter:off
        options.addOption(
            Option.builder(GroupShortCl)
                .desc("Group name where <arg> is the name of the group.  Required and must be unique within the IR.")
                .longOpt(GroupLongCl)
                .hasArg()
                .required()
                .build());
        options.addOption(
            Option.builder(DescShortCl)
                .desc("Description of the group where <arg> is the new description.  Optional.")
                .longOpt(DescLongCl)
                .hasArg()
                .build());
        options.addOption(
            Option.builder(ManagerShortCl)
                .desc("Group administrator where <arg> is the email of the administrator submitter.  Required if adding a new group.  Ignored/optional if adding a member to a group")
                .longOpt(ManagerLongCl)
                .hasArg()
                .build());
        options.addOption(
            Option.builder(MemberShortCl)
                .desc("Submitter to add as a member of the group where <arg> is the email of the submitter to be added.  Optional.")
                .longOpt(MemberLongCl)
                .hasArg()
                .build());
        //@formatter:on
    }
    
    @Override
    protected void customInit()
    {
        String group = null;
        String manager = null;
        String member = null;
        String description = null;
        
        super.customInit();
        StringBuilder initsb = cliBase.getInitsb();
        if(commandline.hasOption(GroupShortCl))
        {
            group = commandline.getOptionValue(GroupShortCl);
            StrH.ttl(initsb, 1, "--", GroupLongCl, " = ", group);
        }
        if(commandline.hasOption(DescShortCl))
        {
            description = commandline.getOptionValue(DescShortCl);
            StrH.ttl(initsb, 1, "--", DescLongCl, " = ", description);
        }
        if(commandline.hasOption(ManagerShortCl))
        {
            manager = commandline.getOptionValue(ManagerShortCl);
            StrH.ttl(initsb, 1, "--", ManagerLongCl, " = ", manager);
        }
        if(commandline.hasOption(MemberShortCl))
        {
            member = commandline.getOptionValue(MemberShortCl);
            StrH.ttl(initsb, 1, "--", MemberLongCl, " = ", member);
        }
        addCmdData = new AddCmdData(group, description, manager, member);
        String msg = addCmdData.validate(commandline);
        if(msg != null)
            help(CliBase.InvalidCommand, msg);
    }
    
    public class AddCmdData
    {
        public final String group;
        public final String manager;
        public final String member;
        public final SubmitterData groupData;
        
        public AddCmdData(String group, String description, String manager, String member)
        {
            this.group = group;
            this.manager = manager;
            this.member = member;
            groupData = new SubmitterData(group, group, description, new Date(), true);
        }
    
        public String validate(CommandLine commandline)
        {
            if(!commandline.hasOption(MemberShortCl) && !commandline.hasOption(ManagerShortCl))
                return "The --" + ManagerLongCl + " is required to add a group.  The --" + MemberLongCl + " is required to add a member to an existing group.";
            return null;
        }
    }
}