package org.opendof.tools.repository.interfaces.cli.cmds.group;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.opendof.tools.repository.interfaces.cli.util.CliBase;
import org.opendof.tools.repository.interfaces.cli.util.CliCommand;
import org.opendof.tools.repository.interfaces.cli.util.StrH;

@SuppressWarnings("javadoc")
public class DeleteGroupCommand extends GroupCommand
{
    // c, h, l, v used
    
    public static final String GroupShortCl = "g";
    public static final String GroupLongCl = "group";

    public static final String MemberShortCl = "m";
    public static final String MemberLongCl = "member";

    public static final String ForceShortCl = "f";
    public static final String ForceLongCl = "force";
    
    private volatile DeleteCmdData deleteCmdData;
    
    public DeleteGroupCommand(CliBase cliBase, CliCommand previousCommand, String commandName)
    {
        super(cliBase, previousCommand, commandName);
    }

    public DeleteCmdData getDeleteCmdData()
    {
        return deleteCmdData;
    }
    
    @Override
    protected void cliSetup()
    {
        super.cliSetup();
        //@formatter:off
        options.addOption(
            Option.builder(GroupShortCl)
                .desc("Group where <arg> is the name of the group.  If given alone the group will be deleted.  --"+ ForceLongCl + " is required to remove a group that has members.  Required.")
                .longOpt(GroupLongCl)
                .hasArg()
                .required()
                .build());
        options.addOption(
            Option.builder(MemberShortCl)
                .desc("Member where <arg> is the name of the member to be removed from the group.  Optional.")
                .longOpt(MemberLongCl)
                .hasArg()
                .build());
        options.addOption(
            Option.builder(ForceShortCl)
                .desc("Force flag, there are no arguments. Required to delete a group that currently has members, otherwise Optional.")
                .longOpt(ForceLongCl)
                .build());
        //@formatter:on
    }
    
    @Override
    protected void customInit()
    {
        super.customInit();
        String group = null;
        String member = null;
        boolean force = false;
        
        StringBuilder initsb = cliBase.getInitsb();
        if (commandline.hasOption(GroupShortCl))
        {
            group = commandline.getOptionValue(GroupShortCl);
            StrH.ttl(initsb, 1, "--", GroupLongCl, " = ", group);
        }
        if (commandline.hasOption(MemberShortCl))
        {
            member = commandline.getOptionValue(MemberShortCl);
            StrH.ttl(initsb, 1, "--", MemberLongCl, " = ", member);
        }
        if(commandline.hasOption(ForceShortCl))
        {
            force = true;
            StrH.ttl(initsb, 1, "--", ForceLongCl, " = ", force);
        }
        deleteCmdData = new DeleteCmdData(group, member, force);
    }


    public class DeleteCmdData
    {
        public final String group;
        public final String member;
        public final boolean force;
        
        public DeleteCmdData(String group, String member, boolean force)
        {
            this.group = group;
            this.member = member;
            this.force = force;
        }
        
        @SuppressWarnings("unused")
        public String validate(CommandLine commandline)
        {
            return null;
        }
    }
}