package org.opendof.tools.repository.interfaces.cli.cmds.group;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.opendof.tools.repository.interfaces.cli.util.CliBase;
import org.opendof.tools.repository.interfaces.cli.util.CliCommand;
import org.opendof.tools.repository.interfaces.cli.util.StrH;

@SuppressWarnings("javadoc")
public class UpdateGroupCommand extends GroupCommand
{
    // c, h, l, v used
    
    public static final String GroupShortCl = "g";
    public static final String GroupLongCl = "group";
    public static final String DescShortCl = "d";
    public static final String DescLongCl = "description";
    public static final String DateShortCl = "t";
    public static final String DateLongCl = "date";
    
    public static final String ManagerShortCl = "a";
    public static final String ManagerLongCl = "group-admin";
    
    private volatile UpdateCmdData updateCmdData;
    
    public UpdateGroupCommand(CliBase cliBase, CliCommand previousCommand, String commandName)
    {
        super(cliBase, previousCommand, commandName);
    }

    public UpdateCmdData getUpdateCmdData()
    {
        return updateCmdData;     
    }
    
    @Override
    protected void cliSetup()
    {
        super.cliSetup();
        //@formatter:off
        options.addOption(
            Option.builder(GroupShortCl)
                .desc("Group to update where <arg> is the name of the group.  Required.")
                .longOpt(DescLongCl)
                .hasArg()
                .required()
                .build());
        options.addOption(
            Option.builder(DescShortCl)
                .desc("Description of the submitter where <arg> is the new description.  Optional.")
                .longOpt(DescLongCl)
                .hasArg()
                .build());
        options.addOption(
            Option.builder(DateShortCl)
                .desc("Creation date where <arg> is in the desired date in mm/dd/yyyy format. Optional")
                .longOpt(DateLongCl)
                .hasArg()
                .build());
        options.addOption(
            Option.builder(ManagerShortCl)
                .desc("The group administrator where <arg> is in the administrator submitter's email.  Required")
                .longOpt(ManagerLongCl)
                .hasArg()
                .build());
        //@formatter:on
    }
    
    @Override
    protected void customInit()
    {
        super.customInit();
        String admin = null;
        String description = null;
        Date date = null;
        String group = null;
        
        StringBuilder initsb = cliBase.getInitsb();
        if(commandline.hasOption(GroupShortCl))
        {
            group = commandline.getOptionValue(GroupShortCl);
            StrH.ttl(initsb, 1, "--", GroupLongCl, " = ", group);
        }
        if(commandline.hasOption(ManagerShortCl))
        {
            admin = commandline.getOptionValue(ManagerShortCl);
            StrH.ttl(initsb, 1, "--", ManagerLongCl, " = ", admin);
        }
        if(commandline.hasOption(DescShortCl))
        {
            description = commandline.getOptionValue(DescShortCl);
            StrH.ttl(initsb, 1, "--", DescLongCl, " = ", description);
        }
        String msg = null;
        if(commandline.hasOption(DateShortCl))
        {
            String dstr = commandline.getOptionValue(DateShortCl);
            SimpleDateFormat sdf = new SimpleDateFormat("MM/dd/yyy");
            try{date = sdf.parse(dstr);}catch(Exception e){msg = "invalid date format";}
            StrH.ttl(initsb, 1, "--", DateLongCl, " = ", dstr);
        }
        if(msg != null)
            help(CliBase.InvalidCommand, msg);
        updateCmdData = new UpdateCmdData(admin, description, date, group);
        msg = updateCmdData.validate(commandline);
        if(msg != null)
            help(CliBase.InvalidCommand, msg);
    }
    
    public class UpdateCmdData
    {
        public final String group;
        public final String admin;
        public final String description;
        public final Date date;
        
        public UpdateCmdData(String admin, String description, Date date, String group) 
        {
            this.group = group;
            this.admin = admin;
            this.description = description;
            this.date = date;
        }
        
        @SuppressWarnings("unused")
        public String validate(CommandLine commandline)
        {
            return null;
        }
    }
}