package org.opendof.tools.repository.interfaces.cli.cmds.iface;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.opendof.tools.repository.interfaces.cli.util.CliBase;
import org.opendof.tools.repository.interfaces.cli.util.CliCommand;
import org.opendof.tools.repository.interfaces.cli.util.StrH;

@SuppressWarnings("javadoc")
public class DeleteIfaceCommand extends IfaceCommand
{
    public static final String RepoShortCl = "r";
    public static final String RepoLongCl = "repository";
    public static final String InterfaceShortCl = "i";
    public static final String InterfaceLongCl = "interface-id";
    public static final String VersionShortCl = "n";
    public static final String VersionLongCl = "version";
    
    public static final String ForceShortCl = "f";
    public static final String ForceLongCl = "force";
    
    private volatile DeleteCmdData deleteCmdData;
    
    public DeleteIfaceCommand(CliBase cliBase, CliCommand previousCommand, String commandName)
    {
        super(cliBase, previousCommand, commandName);
    }

    
    public DeleteCmdData getDeleteCmdData()
    {
        return deleteCmdData;     
    }
    
    
    @Override
    protected void cliSetup()
    {
        super.cliSetup();
        //@formatter:off
        options.addOption(
            Option.builder(RepoShortCl)
                .desc("Repository type. i.e. <opendof || allseen>.  Typically can be determined by the interface ID and is optional in these cases.  Optional.")
                .longOpt(RepoLongCl)
                .hasArg()
                .build());
        options.addOption(
            Option.builder(InterfaceShortCl)
            .desc("An interface or interfaces where <arg> is a space separated list of the Interface IDs.  Required.")
            .longOpt(InterfaceLongCl)
            .hasArgs()
            .required()
            .valueSeparator(' ')
            .build());
        options.addOption(
            Option.builder(VersionShortCl)
                .desc("The version of the given interfaces.  If multiple interfaces are specified this version applies to all of them.  Required.")
                .longOpt(VersionLongCl)
                .required()
                .hasArg()
                .build());
                    
        options.addOption(
            Option.builder(ForceShortCl)
                .desc("Force flag, there are no arguments. Required to delete a published interface, otherwise Optional.")
                .longOpt(ForceLongCl)
                .build());
        //@formatter:on
    }
    
    @Override
    protected void customInit()
    {
        super.customInit();
        
        String repository = null;
        String[] iids = null;
        String version = "1";
        boolean force = false;
        
        StringBuilder initsb = cliBase.getInitsb();
        if(commandline.hasOption(RepoShortCl))
        {
            repository = commandline.getOptionValue(RepoShortCl);
            StrH.ttl(initsb, 1, "--", RepoLongCl, " = ", repository);
        }
        if(commandline.hasOption(InterfaceLongCl))
        {
            iids = commandline.getOptionValues(InterfaceShortCl);
            StrH.ttl(initsb, 1, "--", InterfaceLongCl, " = ", commandline.getOptionValues(InterfaceShortCl));
        }
        if(commandline.hasOption(VersionShortCl))
        {
            version = commandline.getOptionValue(VersionShortCl);
            StrH.ttl(initsb, 1, "--", VersionLongCl, " = ", version);
        }
        
        if(commandline.hasOption(ForceShortCl))
        {
            force = true;
            StrH.ttl(initsb, 1, "--", ForceLongCl, " = ", force);
        }
        
        deleteCmdData = new DeleteCmdData(repository, iids, version, force);
        String msg = deleteCmdData.validate(commandline);
        if(msg != null)
            help(CliBase.InvalidCommand, msg);
    }
    
    public class DeleteCmdData
    {
        public final String repository;
        public final String[] iids;
        public final String version;
        public final boolean force;
        
        public DeleteCmdData(String repository, String[] iids, String version, boolean force)
        {
            this.repository = repository;
            this.iids = iids;
            this.version = version;
            this.force = force;
        }
        
        public String validate(CommandLine commandline)
        {
            if(!commandline.hasOption(InterfaceShortCl))
                return "--" + InterfaceLongCl + " is required";
            if(!commandline.hasOption(VersionShortCl))
                return "--" + VersionLongCl + " is required";
            return null;
        }
    }
}