package org.opendof.tools.repository.interfaces.cli.cmds.permission;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.opendof.tools.repository.interfaces.cli.util.CliBase;
import org.opendof.tools.repository.interfaces.cli.util.CliCommand;
import org.opendof.tools.repository.interfaces.cli.util.StrH;
import org.opendof.tools.repository.interfaces.da.DataAccessor;
import org.opendof.tools.repository.interfaces.da.SubmitterPermissions;

import java.util.Objects;

public class GetPermissionCommand extends PermissionCommand {
    // r, i, n, e used

    public static final String RepoShortCl = "r";
    public static final String RepoLongCl = "repository";
    public static final String InterfaceShortCl = "i";
    public static final String InterfaceLongCl = "interface-id";
    public static final String VersionShortCl = "n";
    public static final String VersionLongCl = "version";
    public static final String EmailShortCl = "e";
    public static final String EmailLongCl = "email";

    private volatile GetPermissionCommand.GetCmdData getCmdData;

    public GetPermissionCommand(CliBase cliBase, CliCommand previousCommand, String commandName)
    {
        super(cliBase, previousCommand, commandName);
    }

    public GetPermissionCommand.GetCmdData getGetCmdData()
    {
        return getCmdData;
    }


    @Override
    protected void cliSetup()
    {
        super.cliSetup();
        //@formatter:off
        options.addOption(
                Option.builder(RepoShortCl)
                        .desc("Repository type. i.e. <opendof || allseen>.  Required.")
                        .longOpt(RepoLongCl)
                        .hasArg()
                        .required()
                        .build());
        options.addOption(
                Option.builder(InterfaceShortCl)
                        .desc("The interface ID. Required.")
                        .longOpt(InterfaceLongCl)
                        .hasArgs()
                        .required()
                        .build());
        options.addOption(
                Option.builder(VersionShortCl)
                        .desc("The version of the given interfaces. Defaults to 1 if not given.")
                        .longOpt(VersionLongCl)
                        .hasArg()
                        .required()
                        .build());
        options.addOption(
                Option.builder(EmailShortCl)
                        .desc("Submitter's email where <arg> is the email which must be unique to the IR. Required and must be unique within the IR.")
                        .longOpt(EmailLongCl)
                        .hasArg()
                        .required()
                        .build());
        //@formatter:on
    }

    @Override
    protected void customInit()
    {
        super.customInit();

        String repository = null;
        String iid = null;
        String version = "1";
        String submitterEmail = null;

        StringBuilder initSB = cliBase.getInitsb();
        if(commandline.hasOption(RepoShortCl))
        {
            repository = commandline.getOptionValue(RepoShortCl);
            StrH.ttl(initSB, 1, "--", RepoLongCl, " = ", repository);
        }
        if(commandline.hasOption(InterfaceLongCl))
        {
            iid = commandline.getOptionValue(InterfaceShortCl);
            StrH.ttl(initSB, 1, "--", InterfaceLongCl, " = ", commandline.getOptionValues(InterfaceShortCl));
        }
        if(commandline.hasOption(VersionShortCl))
        {
            version = commandline.getOptionValue(VersionShortCl);
            StrH.ttl(initSB, 1, "--", VersionLongCl, " = ", version);
        }
        if(commandline.hasOption(EmailShortCl))
        {
            submitterEmail = commandline.getOptionValue(EmailShortCl);
            StrH.ttl(initSB, 1, "--", EmailLongCl, " = ", submitterEmail);
        }

        getCmdData = new GetPermissionCommand.GetCmdData(repository, iid, version, submitterEmail);
        String msg = getCmdData.validate(commandline);
        if(msg != null)
            help(CliBase.InvalidCommand, msg);
    }


    public class GetCmdData
    {
        public final String repository;
        public final String iid;
        public final String version;
        public final String submitterEmail;

        //@formatter:off
        public GetCmdData(
                String _repository, String _iid, String _version,
                String _submitterEmail
        )
        //@formatter:on
        {
            this.repository = _repository;
            this.iid = _iid;
            this.version = _version;
            this.submitterEmail = _submitterEmail;
        }

        public String validate(CommandLine _commandline)
        {
            if(!_commandline.hasOption(InterfaceShortCl))
                return "--" + InterfaceLongCl + " is required";
            if(!_commandline.hasOption(RepoShortCl))
                return "--" + RepoLongCl + " is required";
            if(!_commandline.hasOption(EmailShortCl))
                return "--" + EmailLongCl + " is required";
            return null;
        }
    }

}
