package org.opendof.tools.repository.interfaces.cli.cmds.permission;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.opendof.tools.repository.interfaces.cli.cmds.iface.IfaceCommand;
import org.opendof.tools.repository.interfaces.cli.util.CliBase;
import org.opendof.tools.repository.interfaces.cli.util.CliCommand;
import org.opendof.tools.repository.interfaces.cli.util.StrH;
import org.opendof.tools.repository.interfaces.da.DataAccessor;
import org.opendof.tools.repository.interfaces.da.SubmitterPermissions;

import java.util.Objects;

public class RevokePermissionCommand extends PermissionCommand {

    // r, i, n, e, p used

    public static final String RepoShortCl = "r";
    public static final String RepoLongCl = "repository";
    public static final String InterfaceShortCl = "i";
    public static final String InterfaceLongCl = "interface-id";
    public static final String VersionShortCl = "n";
    public static final String VersionLongCl = "version";
    public static final String EmailShortCl = "e";
    public static final String EmailLongCl = "email";

    private volatile RevokePermissionCommand.RevokeCmdData revokeCmdData;

    public RevokePermissionCommand(CliBase cliBase, CliCommand previousCommand, String commandName)
    {
        super(cliBase, previousCommand, commandName);
    }

    public RevokePermissionCommand.RevokeCmdData getRevokeCmdData()
    {
        return revokeCmdData;
    }


    @Override
    protected void cliSetup()
    {
        super.cliSetup();
        //@formatter:off
        options.addOption(
                Option.builder(RepoShortCl)
                        .desc("Repository type. i.e. <opendof || allseen>.  Required.")
                        .longOpt(RepoLongCl)
                        .hasArg()
                        .required()
                        .build());
        options.addOption(
                Option.builder(InterfaceShortCl)
                        .desc("The interface ID. Required.")
                        .longOpt(InterfaceLongCl)
                        .hasArgs()
                        .required()
                        .build());
        options.addOption(
                Option.builder(VersionShortCl)
                        .desc("The version of the given interfaces. Defaults to 1 if not given.")
                        .longOpt(VersionLongCl)
                        .hasArg()
                        .required()
                        .build());
        options.addOption(
                Option.builder(EmailShortCl)
                        .desc("Submitter's email where <arg> is the email which must be unique to the IR. Required and must be unique within the IR.")
                        .longOpt(EmailLongCl)
                        .hasArg()
                        .required()
                        .build());
        //@formatter:on
    }

    @Override
    protected void customInit()
    {
        super.customInit();

        String repository = null;
        String iid = null;
        String version = "1";
        String submitterEmail = DataAccessor.AdminEmail;
        SubmitterPermissions.PermissionLevel permission = SubmitterPermissions.PermissionLevel.READ;

        StringBuilder initsb = cliBase.getInitsb();
        if(commandline.hasOption(RepoShortCl))
        {
            repository = commandline.getOptionValue(RepoShortCl);
            StrH.ttl(initsb, 1, "--", RepoLongCl, " = ", repository);
        }
        if(commandline.hasOption(InterfaceLongCl))
        {
            iid = commandline.getOptionValue(InterfaceShortCl);
            StrH.ttl(initsb, 1, "--", InterfaceLongCl, " = ", commandline.getOptionValues(InterfaceShortCl));
        }
        if(commandline.hasOption(VersionShortCl))
        {
            version = commandline.getOptionValue(VersionShortCl);
            StrH.ttl(initsb, 1, "--", VersionLongCl, " = ", version);
        }
        if(commandline.hasOption(EmailShortCl))
        {
            submitterEmail = commandline.getOptionValue(EmailShortCl);
            StrH.ttl(initsb, 1, "--", EmailLongCl, " = ", submitterEmail);
        }

        revokeCmdData = new RevokePermissionCommand.RevokeCmdData(repository, iid, version, submitterEmail, permission);
        String msg = revokeCmdData.validate(commandline);
        if(msg != null)
            help(CliBase.InvalidCommand, msg);
    }


    public class RevokeCmdData
    {
        public final String repository;
        public final String iid;
        public final String version;
        public final String submitterEmail;
        public final SubmitterPermissions.PermissionLevel permission;

        //@formatter:off
        public RevokeCmdData(
                String repository, String iid, String version,
                String submitterEmail, SubmitterPermissions.PermissionLevel permission
        )
        //@formatter:on
        {
            this.repository = repository;
            this.iid = iid;
            this.version = version;
            this.permission = permission;
            this.submitterEmail = submitterEmail;
        }

        public String validate(CommandLine commandline)
        {
            if(!commandline.hasOption(InterfaceShortCl))
                return "--" + InterfaceLongCl + " is required";
            if(!commandline.hasOption(VersionShortCl))
                return "--" + VersionLongCl + " is required";
            if(!commandline.hasOption(RepoShortCl))
                return "--" + RepoLongCl + " is required";
            if(!commandline.hasOption(EmailShortCl))
                return "--" + EmailLongCl + " is required";
            return null;
        }
    }

}
