package org.opendof.tools.repository.interfaces.cli.cmds.submitter;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.opendof.tools.repository.interfaces.cli.util.CliBase;
import org.opendof.tools.repository.interfaces.cli.util.CliCommand;
import org.opendof.tools.repository.interfaces.cli.util.StrH;
import org.opendof.tools.repository.interfaces.da.SubmitterData;

@SuppressWarnings("javadoc")
public class AddSubmitterCommand extends SubmitterCommand
{
    // c, d, e, h, l, n, v used
    
    public static final String NameShortCl = "n";
    public static final String NameLongCl = "name";
    public static final String EmailShortCl = "e";
    public static final String EmailLongCl = "email";
    public static final String DescShortCl = "d";
    public static final String DescLongCl = "description";
    
    private volatile AddCmdData addCmdData;
    
    public AddSubmitterCommand(CliBase cliBase, CliCommand previousCommand, String commandName)
    {
        super(cliBase, previousCommand, commandName);
    }

    public AddCmdData getAddCmdData()
    {
        return addCmdData;
    }
    
    @Override
    protected void cliSetup()
    {
        super.cliSetup();
        //@formatter:off
        options.addOption(
            Option.builder(NameShortCl)
                .desc("Submitter's name where <arg> is the name.  Required")
                .longOpt(NameLongCl)
                .hasArg()
                .required()
                .build());
        options.addOption(
            Option.builder(EmailShortCl)
                .desc("Submitter's email where <arg> is the email which must be unique to the IR. Required and must be unique within the IR.")
                .longOpt(EmailLongCl)
                .hasArg()
                .required()
                .build());
        options.addOption(
            Option.builder(DescShortCl)
                .desc("Description of the submitter or group where <arg> is the description.  Optional.")
                .longOpt(DescLongCl)
                .hasArg()
                .build());
        //@formatter:on
    }
    
    @Override
    protected void customInit()
    {
        super.customInit();
        
        String name = null;
        String email = null;
        String description = null;
        
        StringBuilder initsb = cliBase.getInitsb();
        if (commandline.hasOption(NameShortCl))
        {
            name = commandline.getOptionValue(NameShortCl);
            StrH.ttl(initsb, 1, "--", NameLongCl, " = ", name);
        }
        if (commandline.hasOption(EmailShortCl))
        {
            email = commandline.getOptionValue(EmailShortCl);
            StrH.ttl(initsb, 1, "--", EmailLongCl, " = ", email);
        }
        if (commandline.hasOption(DescShortCl))
        {
            description = commandline.getOptionValue(DescShortCl);
            StrH.ttl(initsb, 1, "--", DescLongCl, " = ", description);
        }
        addCmdData = new AddCmdData(name, email, description);
        String msg = addCmdData.validate(commandline);
        if(msg != null)
            help(CliBase.InvalidCommand, msg);
    }
    
    public class AddCmdData
    {
        public final String name;
        public final String email;
        public final String description;
        
        public AddCmdData(String name, String email, String description)
        {
            this.name = name;
            this.email = email;
            this.description = description;
        }
        
        public SubmitterData getSubmitterData()
        {
            return new SubmitterData(name, email, description);
        }
        
        @SuppressWarnings("unused")
        public String validate(CommandLine commandline)
        {
            return null;
        }
    }
}