package org.opendof.tools.repository.interfaces.cli.cmds.submitter;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.opendof.tools.repository.interfaces.cli.util.CliBase;
import org.opendof.tools.repository.interfaces.cli.util.CliCommand;
import org.opendof.tools.repository.interfaces.cli.util.StrH;
import org.opendof.tools.repository.interfaces.da.SubmitterData;

@SuppressWarnings("javadoc")
public class UpdateSubmitterCommand extends SubmitterCommand
{
    // c, d, e, h, l, n, v used
    
    public static final String NameShortCl = "n";
    public static final String NameLongCl = "name";
    public static final String EmailShortCl = "e";
    public static final String EmailLongCl = "email";
    public static final String NewEmailShortCl = "m";
    public static final String NewEmailLongCl = "new-email";
    public static final String DescShortCl = "d";
    public static final String DescLongCl = "description";
    public static final String DateShortCl = "t";
    public static final String DateLongCl = "date";

    private volatile UpdateCmdData updateCmdData;
    
    public UpdateSubmitterCommand(CliBase cliBase, CliCommand previousCommand, String commandName)
    {
        super(cliBase, previousCommand, commandName);
    }

    public UpdateCmdData getUpdateCmdData()
    {
        return updateCmdData;     
    }
    
    @Override
    protected void cliSetup()
    {
        super.cliSetup();
        //@formatter:off
        options.addOption(
            Option.builder(NameShortCl)
                .desc("Submitter's name where <arg> is the new name.  Optional.")
                .longOpt(NameLongCl)
                .hasArg()
                .build());
        options.addOption(
            Option.builder(EmailShortCl)
                .desc("Submitter's email where <arg> is the current email of submitter being updated.  Required.")
                .longOpt(EmailLongCl)
                .hasArg()
                .required()
                .build());
        options.addOption(
            Option.builder(NewEmailShortCl)
                .desc("Submitter's new email where <arg> is the new email of submitter being updated.  Optional.")
                .longOpt(NewEmailLongCl)
                .hasArg()
                .build());
        options.addOption(
            Option.builder(DescShortCl)
                .desc("Description of the submitter where <arg> is the new description.  Optional.")
                .longOpt(DescLongCl)
                .hasArg()
                .build());
        options.addOption(
            Option.builder(DateShortCl)
                .desc("Creation date where <arg> is in the desired date in mm/dd/yyyy format. Optional")
                .longOpt(DateLongCl)
                .hasArg()
                .build());
        //@formatter:on
    }
    
    @Override
    protected void customInit()
    {
        super.customInit();
        
        String name = null;
        String email = null;
        String newEmail = null;
        String description = null;
        Date date = null;
        String msg = null;
        
        StringBuilder initsb = cliBase.getInitsb();
        if (commandline.hasOption(NameShortCl))
        {
            name = commandline.getOptionValue(NameShortCl);
            StrH.ttl(initsb, 1, "--", NameLongCl, " = ", name);
        }
        if (commandline.hasOption(EmailShortCl))
        {
            email = commandline.getOptionValue(EmailShortCl);
            StrH.ttl(initsb, 1, "--", EmailLongCl, " = ", email);
        }
        if (commandline.hasOption(NewEmailShortCl))
        {
            newEmail = commandline.getOptionValue(NewEmailShortCl);
            StrH.ttl(initsb, 1, "--", NewEmailLongCl, " = ", newEmail);
        }
        if (commandline.hasOption(DescShortCl))
        {
            description = commandline.getOptionValue(DescShortCl);
            StrH.ttl(initsb, 1, "--", DescLongCl, " = ", description);
        }
        if(commandline.hasOption(DateShortCl))
        {
            String dstr = commandline.getOptionValue(DateShortCl);
            SimpleDateFormat sdf = new SimpleDateFormat("MM/dd/yyy");
            try{date = sdf.parse(dstr);}catch(Exception e){msg = "invalid date format";}
            StrH.ttl(initsb, 1, "--", DateLongCl, " = ", dstr);
        }
        if(msg != null)
            help(CliBase.InvalidCommand, msg);
        updateCmdData = new UpdateCmdData(name, email, newEmail, description, date);
        msg = updateCmdData.validate(commandline);
        if(msg != null)
            help(CliBase.InvalidCommand, msg);
    }
    
    public class UpdateCmdData
    {
        public final String name;
        public final String email;
        public final String newEmail;
        public final String description;
        public final Date date;
        
        public UpdateCmdData(String name, String email, String newEmail, String description, Date date) 
        {
            this.name = name;
            this.email = email;
            this.newEmail = newEmail;
            this.description = description;
            this.date = date;
        }
        
        public SubmitterData getSubmitterData()
        {
            return new SubmitterData(name, newEmail, description, date);
        }
        
        @SuppressWarnings("unused")
        public String validate(CommandLine commandline)
        {
            return null;
        }
    }
}