package org.opendof.tools.repository.interfaces.cli.cmds.subrepo;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.opendof.tools.repository.interfaces.cli.util.CliBase;
import org.opendof.tools.repository.interfaces.cli.util.CliCommand;
import org.opendof.tools.repository.interfaces.cli.util.StrH;
import org.opendof.tools.repository.interfaces.da.SubRepositoryNode;
import org.opendof.tools.repository.interfaces.da.SubmitterData;

@SuppressWarnings("javadoc")
public class AddSubRepoCommand extends SubRepoCommand
{
    // c, h, l, v used

    public static final String FdnShortCl = "p";
    public static final String FdnLongCl = "path";
    public static final String LabelShortCl = "b";
    public static final String LabelLongCl = "label";
    public static final String GroupShortCl = "g";
    public static final String GroupLongCl = "group";

    private volatile AddCmdData addCmdData;

    public AddSubRepoCommand(CliBase cliBase, String commandName)
    {
        super(cliBase, commandName);
    }

    public AddSubRepoCommand(CliBase cliBase, CliCommand previousCommand, String commandName)
    {
        super(cliBase, previousCommand, commandName);
    }

    public AddCmdData getAddCmdData()
    {
        return addCmdData;
    }

    @Override
    protected void cliSetup()
    {
        super.cliSetup();
        //@formatter:off
        options.addOption(
            Option.builder(FdnShortCl)
                .desc("Full path name of the node to be added where <arg> is a '/' seperated path with either opendof | allseen as the root.  For example 'opendof/1/4 would expect 'opendof/1' to already exist and 4 will be added as a child to it.  Required")
                .longOpt(FdnLongCl)
                .hasArg()
                .required()
                .build());
        options.addOption(
            Option.builder(LabelShortCl)
                .desc("The human friendly label to be associated with the node.  Required")
                .longOpt(LabelLongCl)
                .hasArg()
                .required()
                .build());
        options.addOption(
            Option.builder(GroupShortCl)
                .desc("The access group to be associated with the node where <arg> is the name of an existing group in the IR.  Optional")
                .longOpt(GroupLongCl)
                .hasArg()
                .build());
        //@formatter:on
    }

    @Override
    protected void customInit()
    {
        String fdn = null;
        String label = null;
        String group = null;
        
        StringBuilder initsb = cliBase.getInitsb();
        if (commandline.hasOption(FdnShortCl))
        {
            fdn = commandline.getOptionValue(FdnShortCl);
            StrH.ttl(initsb, 1, "--", FdnLongCl, " = " + fdn);
        }
        if (commandline.hasOption(LabelShortCl))
        {
            label = commandline.getOptionValue(LabelShortCl);
            StrH.ttl(initsb, 1, "--", LabelLongCl, " = " + label);
        }
        if (commandline.hasOption(GroupShortCl))
        {
            group = commandline.getOptionValue(GroupShortCl);
            StrH.ttl(initsb, 1, "--", GroupLongCl, " = " + group);
        }

        addCmdData = new AddCmdData(fdn, label, group);
        String msg = addCmdData.validate(commandline);
        if (msg != null)
            help(CliBase.InvalidCommand, msg);
    }

    public class AddCmdData
    {
        public final String rdn;
        public final SubRepositoryNode child;

        public AddCmdData(String fdn, String label, String group)
        {
            String name = StrH.getAtomicName(fdn, '/');
            int idx = fdn.indexOf('/');
            String repoType = fdn.substring(0, idx);
            String runningRdn = fdn.substring(idx+1);
            StringBuilder rdnb = new StringBuilder();
            int depth = 1;
            do
            {
                idx = runningRdn.indexOf('/');
                if(idx == -1)
                    break;
                rdnb.append(runningRdn.substring(0, idx));
                runningRdn = runningRdn.substring(idx + 1);
                ++depth;
            }while(true);
            child = new SubRepositoryNode(repoType, name, label, depth, group == null ? null : new SubmitterData(group, group, null));
            rdn = rdnb.length() == 0 ? null : rdnb.toString();
        }

        @SuppressWarnings("unused")
        public String validate(CommandLine commandline)
        {
            return null;
        }
    }
}