package org.opendof.tools.repository.interfaces.cli.cmds.subrepo;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.opendof.tools.repository.interfaces.cli.util.CliBase;
import org.opendof.tools.repository.interfaces.cli.util.CliCommand;
import org.opendof.tools.repository.interfaces.cli.util.StrH;
import org.opendof.tools.repository.interfaces.da.SubRepositoryNode;

@SuppressWarnings("javadoc")
public class DeleteSubRepoCommand extends SubRepoCommand
{
    // c, h, l, v used
    public static final String FdnShortCl = "p";
    public static final String FdnLongCl = "path";
    public static final String LabelShortCl = "b";
    public static final String LabelLongCl = "label";
    
    private volatile DeleteCmdData deleteCmdData;
    
    public DeleteSubRepoCommand(CliBase cliBase, CliCommand previousCommand, String commandName)
    {
        super(cliBase, previousCommand, commandName);
    }

    public DeleteCmdData getDeleteCmdData()
    {
        return deleteCmdData;
    }
    
    @Override
    protected void cliSetup()
    {
        super.cliSetup();
        //@formatter:off
        options.addOption(
            Option.builder(FdnShortCl)
                .desc("Full path name of the node to be deleted where <arg> is a '/' seperated path with either opendof | allseen as the root.  For example 'opendof/1/4 would delete 4 and leave 'opendof/1'.  Required")
                .longOpt(FdnLongCl)
                .hasArg()
                .required()
                .build());
        options.addOption(
            Option.builder(LabelShortCl)
                .desc("The human friendly label associated with the node.  Required")
                .longOpt(LabelLongCl)
                .hasArg()
                .required()
                .build());
        //@formatter:on
    }
    
    @Override
    protected void customInit()
    {
        String fdn = null;
        String label = null;

        StringBuilder initsb = cliBase.getInitsb();
        if(commandline.hasOption(FdnShortCl))
        {
            fdn = commandline.getOptionValue(FdnShortCl);
            StrH.ttl(initsb, 1, "--", FdnLongCl, " = " + fdn);
        }
        if (commandline.hasOption(LabelShortCl))
        {
            label = commandline.getOptionValue(LabelShortCl);
            StrH.ttl(initsb, 1, "--", LabelLongCl, " = " + label);
        }
        deleteCmdData = new DeleteCmdData(fdn, label);
        String msg = deleteCmdData.validate(commandline);
        if(msg != null)
            help(CliBase.InvalidCommand, msg);
    }
    
    public class DeleteCmdData
    {
        public final SubRepositoryNode subrepo;
        public final String parentRdn;

        public DeleteCmdData(String fdn, String label)
        {
            String name = StrH.getAtomicName(fdn, '/');
            int idx = fdn.indexOf('/');
            String repoType = fdn.substring(0, idx);
            String runningRdn = fdn.substring(idx+1);
            StringBuilder rdnb = new StringBuilder();
            int depth = 1;
            do
            {
                idx = runningRdn.indexOf('/');
                if(idx == -1)
                    break;
                rdnb.append(runningRdn.substring(0, idx));
                runningRdn = runningRdn.substring(idx + 1);
                ++depth;
            }while(true);
            subrepo = new SubRepositoryNode(repoType, name, label, depth, null);
            parentRdn = rdnb.length() == 0 ? null : rdnb.toString();
        }

        @SuppressWarnings("unused")
        public String validate(CommandLine commandline)
        {
            return null;
        }
    }
}