package org.opendof.tools.repository.interfaces.cli.cmds.subrepo;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.opendof.tools.repository.interfaces.cli.util.CliBase;
import org.opendof.tools.repository.interfaces.cli.util.CliCommand;
import org.opendof.tools.repository.interfaces.cli.util.StrH;

@SuppressWarnings("javadoc")
public class ListSubRepoCommand extends SubRepoCommand
{
    // c, h, l, v used
    
    public static final String FdnShortCl = "p";
    public static final String FdnLongCl = "path";
    
    private volatile ListCmdData listCmdData;
    
    public ListSubRepoCommand(CliBase cliBase, CliCommand previousCommand, String commandName)
    {
        super(cliBase, previousCommand, commandName);
    }

    public ListCmdData getListCmdData()
    {
        return listCmdData;
    }
    
    @Override
    protected void cliSetup()
    {
        super.cliSetup();
        //@formatter:off
        options.addOption(
            Option.builder(FdnShortCl)
                .desc("Full path name of the node to be listed from where <arg> is a '/' seperated path with either opendof | allseen as the root.  For example 'opendof' would list the whole tree and 'opendof/1' would list registry 1.  At least the root node is required")
                .longOpt(FdnLongCl)
                .hasArg()
                .required()
                .build());
        //@formatter:on
    }
    
    @Override
    protected void customInit()
    {
        String fdn = null;

        StringBuilder initsb = cliBase.getInitsb();
        if(commandline.hasOption(FdnShortCl))
        {
            fdn = commandline.getOptionValue(FdnShortCl);
            StrH.ttl(initsb, 1, "--", FdnLongCl, " = " + fdn);
        }
        listCmdData = new ListCmdData(fdn);
        String msg = listCmdData.validate(commandline);
        if(msg != null)
            help(CliBase.InvalidCommand, msg);
    }
    
    public class ListCmdData
    {
        public final String fdn;
        public final String repoType;
        public final String rdn;

        public ListCmdData(String fdn)
        {
            this.fdn = fdn;
            int idx = fdn.indexOf('/');
            if(idx == -1)
                repoType = fdn;
            else
                repoType = fdn.substring(0, idx);
            rdn = fdn.substring(idx+1);
        }

        @SuppressWarnings("unused")
        public String validate(CommandLine commandline)
        {
            return null;
        }
    }
}