package org.opendof.tools.repository.interfaces.core;

import java.io.Writer;

public interface XMLTranslator {

	/**
	 * Translate an XML string to a translated HTML-formatted string and write to the writer object.
	 * 
	 * @param xml The source XML string.
	 * @param writer The object to which the translated string is written.
	 * @throws Exception Thrown if any exceptions occur during translation.
	 */
	public void toHTML(String xml, Writer writer) throws Exception;
	
	/**
	 * Translate an XML string to a raw(non-html formatted) string and write to the writer object.
	 * 
	 * @param xml The source XML string.
	 * @param writer The object to which the translated string is written.
	 * @return The HTTP Content-Type to use when sending the response.
	 * @throws Exception Thrown if any exceptions occur during translation.
	 */
	public String toRaw(String xml, Writer writer) throws Exception;
	
	/**
	 * Whether or not this translation of the interface definition supports multiple languages.(Using html lang="" attribute).
	 *  
	 * @return True if multiple languages are supported. False otherwise.
	 */
	public boolean hasLanguageSupport();
	
	/**
	 * Whether or not the translator supports a Raw format(not html formatted).
	 * @return True if the translator supports non-html formatted translation.
	 */
	public boolean hasRawSupport();
	
	/**
	 * Whether or not the translator supports an HTML format.
	 * @return True if the translator supports html formatted translation.
	 */
	public boolean hasHTMLSupport();
	
	/**
	 * Get the name of this translation.
	 * @return The name of this translation.
	 */
	public String getTranslationName();
}
