/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/
package org.opendof.tools.repository.interfaces.core;

import java.io.BufferedReader;
import java.io.Reader;
import java.io.StringReader;
import java.io.Writer;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Translates xml using xslt.
 *
 */
public class XSLTTranslator implements XMLTranslator
{
    private static Logger log = LoggerFactory.getLogger(XSLTTranslator.class);
    
    private String xslt;
    private String name;
    
    /**
     * Creates an XSLTTranslator object using the specified xslt file.
     * 
     * @param xslt The xslt filename.
     * @param name The name of this translation
     */
    public XSLTTranslator(String xslt, String name){
    	this.xslt = xslt;
    	this.name = name;
    }

    @Override
    public void toHTML(String xml, Writer writer) throws Exception
    {
    	if(xml == null){
    		return;
    	}
        log.debug(XSLTTranslator.class.getSimpleName() + ".translate with: " + xslt);
        Reader reader = new BufferedReader(new StringReader(xml));
        try
        {
            TransformerFactory tf = TransformerFactory.newInstance();
            Transformer transformer = tf.newTransformer(new StreamSource(xslt));
            transformer.transform(new StreamSource(reader), new StreamResult(writer));
        }
        finally
        {
            reader.close();
        }
    }

	@Override
	public boolean hasLanguageSupport() {
		return true;
	}

	@Override
	public String toRaw(String xml, Writer writer) throws Exception {
		return null;
	}

	@Override
	public boolean hasRawSupport() {
		return false;
	}

	@Override
	public String getTranslationName() {
		return name;
	}

	@Override
	public boolean hasHTMLSupport() {
		return true;
	}
}
