package org.opendof.tools.repository.interfaces.mysql;

import org.opendof.tools.repository.interfaces.da.NotFoundException;
import org.opendof.tools.repository.interfaces.da.SubmitterException;
import org.opendof.tools.repository.interfaces.da.SubmitterPermissions;
import org.opendof.tools.repository.interfaces.mysql.data.MysqlPermissionsIterable;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class InterfacePermissionJdbc {
    public static final String TableName;

    public static final String ColUserFkName;
    public static final String ColIfaceFkName;
    public static final String ColPermissionName;

    public static final int ColPermissionNameIdx = 7;

    private static final String PrepAddUserInterfacePermissions;
    private static final String PrepDeleteUserInterfacePermissions;
    private static final String PrepGetPermissions;
    private static final String PrepGetPermission;

    static
    {
        TableName = "interfacepermissions";

        ColUserFkName = "userFk";
        ColIfaceFkName = "ifaceFk";
        ColPermissionName = "accesspermission";

        //@formatter:off
        PrepAddUserInterfacePermissions = "INSERT INTO " + TableName +
            "(" +
                ColUserFkName + ", " +
                ColIfaceFkName + ", " +
                ColPermissionName +
            ") " +
            " VALUES (" +
                " (SELECT " + SubmitterJdbc.ColSubmitterPkName +
                    " FROM " + SubmitterJdbc.TableName +
                    " WHERE " + SubmitterJdbc.ColEmailName +
                    " = ? AND " + SubmitterJdbc.ColGroupName +
                    " = FALSE LIMIT 1)," +
                " (SELECT "+ InterfaceJdbc.ColIfacePkName +
                    " FROM " + InterfaceJdbc.TableName +
                    " WHERE " + InterfaceJdbc.ColRepoTypeName + " = ? " +
                    " AND " + InterfaceJdbc.ColIidName + " = ? " +
                    " AND " + InterfaceJdbc.ColVersionName +
                    " = ? " + " LIMIT 1)," +
            " ?)" +
            " ON DUPLICATE KEY UPDATE " + ColPermissionName + "= ?;";

        PrepDeleteUserInterfacePermissions = "DELETE FROM " + TableName +
            " WHERE " + ColUserFkName + " = (SELECT " + SubmitterJdbc.ColSubmitterPkName +" FROM " + SubmitterJdbc.TableName +" WHERE " + SubmitterJdbc.ColEmailName + " = ? AND " + SubmitterJdbc.ColGroupName + " = FALSE LIMIT 1)" +
            " AND " + ColIfaceFkName + " = (SELECT "+ InterfaceJdbc.ColIfacePkName +
                " FROM " + InterfaceJdbc.TableName +
                " WHERE " + InterfaceJdbc.ColRepoTypeName + " = ? " +
                " AND " + InterfaceJdbc.ColIidName + " = ? " +
                " AND " + InterfaceJdbc.ColVersionName + " = ? " + " LIMIT 1)";


        PrepGetPermissions = "SELECT " +
                SubmitterJdbc.ColSubmitterPkName + ", " +
                SubmitterJdbc.ColSubmitterName + ", " +
                SubmitterJdbc.ColEmailName + ", " +
                SubmitterJdbc.ColDescName + ", " +
                SubmitterJdbc.ColDateName + ", " +
                SubmitterJdbc.ColGroupName + ", " +
                ColPermissionName +
            " FROM " + TableName +  " INNER JOIN " + SubmitterJdbc.TableName + " ON " + SubmitterJdbc.ColSubmitterPkName + " = " + ColUserFkName +
            " WHERE " + ColIfaceFkName + " = (SELECT "+ InterfaceJdbc.ColIfacePkName +
                " FROM " + InterfaceJdbc.TableName +
                " WHERE " + InterfaceJdbc.ColRepoTypeName + " = ? " +
                " AND " + InterfaceJdbc.ColIidName + " = ? " +
                " AND " + InterfaceJdbc.ColVersionName + " = ? " + " LIMIT 1);";

        PrepGetPermission = "SELECT " + ColPermissionName + " FROM " + TableName +
            " WHERE " + ColIfaceFkName + " = " + " (SELECT "+ InterfaceJdbc.ColIfacePkName +
                " FROM " + InterfaceJdbc.TableName +
                " WHERE " + InterfaceJdbc.ColRepoTypeName + " = ? " +
                " AND " + InterfaceJdbc.ColIidName + " = ? " +
                " AND " + InterfaceJdbc.ColVersionName + " = ? " + " LIMIT 1) " +
            " AND " + ColUserFkName + " = " + "(SELECT " + SubmitterJdbc.ColSubmitterPkName +" FROM " + SubmitterJdbc.TableName +" WHERE " + SubmitterJdbc.ColEmailName + " = ? AND " + SubmitterJdbc.ColGroupName + " = FALSE LIMIT 1)";
        //@formatter:on
    }

    public static void addPermission(Connection connection, String repoType, String iid, String version, String granteeEmail, int permission) throws NotFoundException, SubmitterException, SQLException {
        PreparedStatement stmt;
        stmt = connection.prepareStatement(PrepAddUserInterfacePermissions);

        try
        {
            stmt.setString(1, granteeEmail);
            stmt.setString(2, repoType);
            stmt.setString(3, iid);
            stmt.setString(4, version);
            stmt.setInt(5, permission);
            stmt.setInt(6, permission);
            stmt.executeUpdate();

        } finally
        {
            MysqlDataAccessor.close(connection, stmt, null, true);
        }
    }

    public static void revokePermissions(Connection connection, String repoType, String iid, String version, String granteeEmail) throws NotFoundException, Exception {
        PreparedStatement stmt;
        stmt = connection.prepareStatement(PrepDeleteUserInterfacePermissions);

        try
        {
            stmt.setString(1, granteeEmail);
            stmt.setString(2, repoType);
            stmt.setString(3, iid);
            stmt.setString(4, version);
            int rows = stmt.executeUpdate();
            if (rows != 1)
                throw new SQLException("revoke permission affected an unexpected number of rows: " + rows);
        } finally
        {
            MysqlDataAccessor.close(connection, stmt, null, true);
        }
    }

    public static List<SubmitterPermissions> getPermissions(Connection connection, String repoType, String iid, String version) throws NotFoundException, Exception {

        PreparedStatement stmt = connection.prepareStatement(PrepGetPermissions);
        stmt.setString(1, repoType);
        stmt.setString(2, iid);
        stmt.setString(3, version);

        ResultSet rs = null;

        try
        {
            rs = stmt.executeQuery();
            MysqlPermissionsIterable subiter = new MysqlPermissionsIterable(connection, stmt, rs);
            Iterator<SubmitterPermissions> siter = subiter.iterator();
            List<SubmitterPermissions> permissions = new ArrayList<>();
            while(siter.hasNext())
            {
                permissions.add(siter.next());
            }
            subiter.close();
            return permissions;
        }catch(Exception e)
        {
            MysqlDataAccessor.close(connection, stmt, rs, true);
            throw e;
        }
    }

    public static Integer getPermission(Connection connection, String repoType, String iid, String version, String granteeEmail) throws NotFoundException, Exception {
        PreparedStatement stmt;
        stmt = connection.prepareStatement(PrepGetPermission);

        try
        {
            stmt.setString(1, repoType);
            stmt.setString(2, iid);
            stmt.setString(3, version);
            stmt.setString(4, granteeEmail);
            ResultSet rs  = stmt.executeQuery();
            if (!rs.next())
                throw new NotFoundException("permission for iid: " + iid + " user: " + granteeEmail + " not found");
            return rs.getInt(1);
        } finally
        {
            MysqlDataAccessor.close(connection, stmt, null, true);
        }
    }



}
