
package org.opendof.tools.repository.interfaces.mysql.data;

import org.apache.commons.dbutils.ResultSetIterator;
import org.opendof.tools.repository.interfaces.da.SubmitterPermissions;
import org.opendof.tools.repository.interfaces.da.DataAccessor.SubmitterPermissionsIterable;
import org.opendof.tools.repository.interfaces.mysql.MysqlDataAccessor;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Iterator;
import java.util.concurrent.atomic.AtomicBoolean;


public class MysqlPermissionsIterable implements SubmitterPermissionsIterable
{
    private final SubmitterPermissionsIterator creatorIterator;

    public MysqlPermissionsIterable(Connection connection, PreparedStatement stmt, ResultSet rs)
    {
        creatorIterator = new SubmitterPermissionsIterator(connection, stmt, rs);
    }

    @Override
    public void close() throws Exception
    {
        creatorIterator.close();
    }

    @Override
    public Iterator<SubmitterPermissions> iterator()
    {
        return creatorIterator;
    }

    private class SubmitterPermissionsIterator implements Iterator<SubmitterPermissions>
    {
        private final ResultSetIterator rsi;
        private final Connection connection;
        private final PreparedStatement stmt;
        private final ResultSet rs;
        private final AtomicBoolean closed;


        private SubmitterPermissionsIterator(Connection connection, PreparedStatement stmt, ResultSet rs)
        {
            this.connection = connection;
            this.stmt = stmt;
            this.rs = rs;
            rsi = new ResultSetIterator(rs);
            closed = new AtomicBoolean(false);
        }

        @Override
        public boolean hasNext()
        {
            if (closed.get())
                throw new IllegalStateException("This iterator has been closed");
            return rsi.hasNext();
        }

        @Override
        public SubmitterPermissions next()
        {
            if (closed.get())
                throw new IllegalStateException("This iterator has been closed");
            Object[] columns = rsi.next();
            return new SubmitterPermissionRow(columns);
        }

        public void close() throws Exception
        {
            if (closed.get())
                return;
            closed.set(true);
            MysqlDataAccessor.close(connection, stmt, rs, true);
        }
    }

}
