package org.opendof.tools.repository.interfaces.opendof.iface.compute;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.opendof.tools.repository.interfaces.opendof.CLanguageTranslator;
import org.opendof.tools.repository.interfaces.opendof.saxParser.MetadataElements;
import org.opendof.tools.repository.interfaces.opendof.saxParser.method.MethodElement;
import org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.collection.FieldData;

public class MethodInstance {
    // Object Access protocol specification: Compressed Unsigned 16 bits, which equates to 2**15 max; can use Java short
	private final short itemId;
	
    private Integer [] xmlBasedInputTypeReferences;
    private Integer [] xmlBasedOutputTypeReferences;
    private short [] codeBlockInputTypeReferences;
    private short [] codeBlockOutputTypeReferences;

    // metadata, considered optional; these defaults are generally overwritten by processing xml-supplied data
    private String codename = "";
    private String name = "";
    private String description = "";

    public MethodInstance(CLanguageTranslator clt, MethodElement methodElement,  String strRequestedLanguage) throws Exception {
    	this.itemId = (short) methodElement.itemId;

        List<FieldData> inputs = methodElement.getInputs();
        List<Integer> typeRefs = new ArrayList<Integer>();
        for (FieldData fd : inputs)
        	typeRefs.add(fd.getReference());
        this.xmlBasedInputTypeReferences = Arrays.copyOf(typeRefs.toArray(), typeRefs.size(), Integer[].class);
    	
        List<FieldData> outputs = methodElement.getOutputs();
        typeRefs = new ArrayList<Integer>();
        for (FieldData fd : outputs)
        	typeRefs.add(fd.getReference());
        this.xmlBasedOutputTypeReferences = Arrays.copyOf(typeRefs.toArray(), typeRefs.size(), Integer[].class);

        this.codeBlockInputTypeReferences = new short[this.xmlBasedInputTypeReferences.length];
        for (int i=0; i<this.xmlBasedInputTypeReferences.length; i++) {
        	short xmlBasedTypeReference = this.xmlBasedInputTypeReferences[i].shortValue(); 
        	this.codeBlockInputTypeReferences[i] = clt.getCodeBlockTypeReference(xmlBasedTypeReference);
        }
        
        this.codeBlockOutputTypeReferences = new short[this.xmlBasedOutputTypeReferences.length];
        for (int i=0; i<this.xmlBasedOutputTypeReferences.length; i++) {
        	short xmlBasedTypeReference = this.xmlBasedOutputTypeReferences[i].shortValue(); 
        	this.codeBlockOutputTypeReferences[i] = clt.getCodeBlockTypeReference(xmlBasedTypeReference);
        }
        
    	// where possible, override default metadata codename, name, and description
    	MetadataElements mElements = methodElement.getMetadata();    	
    	if (mElements != null) {
    		String localCodeName = mElements.getCodeName(); // e.g. "Status"
    		this.codename = localCodeName!=null ? localCodeName : "Method_" + Form.nextDistinguishingCharacter();
    		String localName = Form.getInterfaceElementDisplayName(mElements, strRequestedLanguage); // e.g. "Status"
    		if (localName != null)
    			this.name = localName;
    		String localDescription = Form.getInterfaceElementDescription(mElements, strRequestedLanguage); // e.g. "This is ..."
    		if (localDescription != null)
    			this.description = localDescription;
    	}
        this.computeDetail();
    }

	private void computeDetail() throws Exception {

	}
	
	public short getItemId() {
		return this.itemId;
	}
	
    public String getCodeName() {
    	return this.codename;
    }

    public Integer [] getXmlBasedInputTypeReferences() {
    	return this.xmlBasedInputTypeReferences;
    }
    
    public Integer [] getXmlBasedOutputTypeReferences() {
    	return this.xmlBasedOutputTypeReferences;
    }
    
    public short [] getCodeBlockInputTypeReferences() {
    	return this.codeBlockInputTypeReferences;
    }

    public short [] getCodeBlockOutputTypeReferences() {
    	return this.codeBlockOutputTypeReferences;
    }
    
    public String getName() {
    	return this.name;
    }
    
    public String getDescription() {
    	return this.description;
    }

}
