package org.opendof.tools.repository.interfaces.opendof.iface.compute;

import org.opendof.tools.repository.interfaces.opendof.CLanguageTranslator;
import org.opendof.tools.repository.interfaces.opendof.saxParser.MetadataElements;
import org.opendof.tools.repository.interfaces.opendof.saxParser.property.PropertyElement;

/**
 * Temporary storage for a property definition.
 * 
 * Note: the reference number used to locate this property's type is not stored (supplied in the interface definition)
 */
public class PropertyInstance {
    // Object Access protocol specification: Compressed Unsigned 16 bits, which equates to 2**15 max; can use Java short
	private final short itemId;
	
    private final boolean readable;
    private final boolean writeable;
    private final short codeBlockTypeReference;
    private final String typename;
    
    // metadata, considered optional; these defaults are generally overwritten by processing xml-supplied data
    private String codename = "";
    private String name = "";
    private String description = "";

    public PropertyInstance(CLanguageTranslator clt, PropertyElement propertyElement, String strRequestedLanguage) throws Exception {
    	this.itemId = (short)propertyElement.itemId;
    	this.readable = propertyElement.isReadable();
    	this.writeable = propertyElement.isWriteable();
    	short typeref = (short)propertyElement.reference;
    	this.typename = clt.getMapTypeInstances().get(typeref).getCodeName(); // e.g. "Status Type"
    	this.codeBlockTypeReference = clt.getCodeBlockTypeReference(typeref);
    	
    	// where possible, override default metadata codename, name, and description
    	MetadataElements mElements = propertyElement.getMetadata();    	
    	if (mElements != null) {
    		String localCodeName = mElements.getCodeName(); // e.g. "Status"
    		this.codename = localCodeName!=null ? localCodeName : "Property_" + Form.nextDistinguishingCharacter();
    		String localName = Form.getInterfaceElementDisplayName(mElements, strRequestedLanguage); // e.g. "Status"
    		if (localName != null)
    			this.name = localName;
    		String localDescription = Form.getInterfaceElementDescription(mElements, strRequestedLanguage); // e.g. "This is ..."
    		if (localDescription != null)
    			this.description = localDescription;
    	}
        this.computeDetail();
    }
    
	private void computeDetail() throws Exception {

	}

	public boolean getReadable() {
		return this.readable;
	}

	public boolean getWriteable() {
		return this.writeable;
	}

	public short getCodeBlockTypeReference() {
		return this.codeBlockTypeReference;
	}
	
    public short getItemId() {
    	return this.itemId;
    }
    
    public String getCodeName() {
    	return this.codename;
    }

    public String getTypeName() {
    	return this.typename;
    }
    
    public String getName() {
    	return this.name;
    }
    
    public String getDescription() {
    	return this.description;
    }
    
}