/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/
package org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.collection;

import org.opendof.tools.repository.interfaces.opendof.saxParser.MetadataElements;
import org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.DofTypeBase;
import org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.TypedefsElement;
import org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.reference.DofReference;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * DofArray Element SAX handler.  
 */
public class DofArrayElement extends DofReference
{
    private static String ArrayElement = "array";

	/** The minimum length of this array, null if not specified */
    public volatile Integer minLength;
	/** The length of this array */
    public volatile int length;
    
    /**
     * DofArrayElement constructor
     * @param childComplete the child completion callback to my parent.  Must not be null.
     */
    public DofArrayElement(ChildComplete childComplete)
    {
        super(childComplete);
    }

    public DofArrayElement(int itemId, int reference, MetadataElements metadata, Integer minLength, int length)
    {
        super(itemId, reference, metadata);
        this.minLength = minLength;
        this.length = length;
    }

    public Integer getMinLength()
    {
        return minLength;
    }
    
    public int getLength()
    {
        return length;
    }
    
    @Override
    public void validate(TypedefsElement typedefs) throws SAXException
    {
        super.validate(typedefs);
        if (minLength != null && minLength > length)
            throw new SAXException("minLength > length,  interface.typedefs." + getClass().getSimpleName());
    }

    @Override
    protected int setAttributes(Attributes attributes) throws SAXException
    {
        int size = super.setAttributes(attributes);
        if(length == 1)
            throw new SAXException(currentPath() + " DofArray with no attributes");
        if (size > 4)
            throw new SAXException(currentPath() + " DofArray with too many attributes");
        int index = attributes.getIndex(LengthAttr);
        if (index < 0)
            throw new SAXException(currentPath() + " DofArray attribute not found: " + LengthAttr);
        length = Integer.parseInt(attributes.getValue(index));
        if (size == 4)
        {
            index = attributes.getIndex(MinLengthAttr);
            if (index < 0)
                throw new SAXException(currentPath() + " DofArray attribute not found: " + MinLengthAttr);
            minLength = Integer.parseInt(attributes.getValue(index));
        }else
            minLength = length;
        return size;
    }

    @Override
    public StringBuilder export(StringBuilder sb, int level)
    {
        if (minLength != null)
            //@formatter:off
            element(sb, ArrayElement, 
                            new String[]{TypeIdAttr, FieldData.TypeRefAttr, MinLengthAttr, LengthAttr}, 
                            new String[]{""+typeId, ""+reference, minLength.toString(), ""+length}, 
                            null, metadata == null, level);
            //@formatter:on
        else
            element(sb, ArrayElement, new String[] { TypeIdAttr, FieldData.TypeRefAttr, LengthAttr }, new String[] {""+typeId, ""+reference, ""+length }, null, metadata == null, level);
        return exportNames(sb, ArrayElement, metadata, true, level); 
//        ++level;
//        FieldData field = new FieldData(null);
//        field.metadata = metadata;
//        field.reference = reference;
//        field.export(sb, level);
//        --level;
//        return element(sb, ArrayElement, CType.Close, level);
    }
    
    @Override
    public boolean validPublish(DofTypeBase doftype)
    {
        if(!super.validPublish(doftype))
            return false;
        if(!checkEquals(minLength, ((DofArrayElement)doftype).minLength))
            return false;
        if(length != ((DofArrayElement)doftype).length)
            return false;
        return true;
    }
}
