/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/
package org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.collection;

import org.opendof.tools.repository.interfaces.opendof.saxParser.MetadataElements;
import org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.DofTypeBase;
import org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.TypedefsElement;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * DofBlob Element SAX handler.  
 */
@SuppressWarnings("javadoc")
public class DofBlobElement extends DofTypeBase
{
    private static String BlobElement = "blob";

    public Integer minLength;
    public int length;

    /**
     * DofBlobElement constructor
     * @param childComplete the child completion callback to my parent.  Must not be null.
     */
    public DofBlobElement(ChildComplete childComplete)
    {
        super(childComplete);
    }

    public DofBlobElement(int itemId, MetadataElements metadata, Integer minLength, int length)
    {
        super(itemId, metadata);
        this.minLength = minLength;
        this.length = length;
    }

    public Integer getMinLength()
    {
        return minLength;
    }
    
    public int getLength()
    {
        return length;
    }
    
    @Override
    public void validate(TypedefsElement typedefs) throws SAXException
    {
        super.validate(typedefs);
        if (minLength != null && minLength > length)
            throw new SAXException("minLength > length,  interface.typedefs." + getClass().getSimpleName());
    }

    @Override
    protected int setAttributes(Attributes attributes) throws SAXException
    {
        int size = super.setAttributes(attributes);
        if(length == 1)
            throw new SAXException(currentPath() + " DofBlob with no attributes");
        if (size > 3)
            throw new SAXException(currentPath() + " DofBlob with too many attributes");
        int index = attributes.getIndex(LengthAttr);
        if (index < 0)
            throw new SAXException(currentPath() + " DofBlob attribute not found: " + LengthAttr);
        length = Integer.parseInt(attributes.getValue(index));
        if (size == 3)
        {
            index = attributes.getIndex(MinLengthAttr);
            if (index < 0)
                throw new SAXException(currentPath() + " DofBlob attribute not found: " + MinLengthAttr);
            minLength = Integer.parseInt(attributes.getValue(index));
        }else
            minLength = length;
        return size;
    }

    @Override
    public StringBuilder export(StringBuilder sb, int level)
    {
        if (minLength != null)
            element(sb, BlobElement, new String[] { TypeIdAttr, MinLengthAttr, LengthAttr }, new String[] { ""+typeId, minLength.toString(), "" + length }, null, metadata == null, level);
        else
            element(sb, BlobElement, new String[] { TypeIdAttr, LengthAttr }, new String[] { ""+typeId, "" + length }, null, metadata == null, level);
        return exportNames(sb, BlobElement, metadata, true, level);
    }
    
    @Override
    public boolean validPublish(DofTypeBase doftype)
    {
        if(!super.validPublish(doftype))
            return false;
        if(!checkEquals(minLength, ((DofArrayElement)doftype).minLength))
            return false;
        if(length != ((DofArrayElement)doftype).length)
            return false;
        return true;
    }
}
