/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/
package org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.numerical;

import org.opendof.tools.repository.interfaces.opendof.saxParser.MetadataElements;
import org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.TypedefsElement;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * DofInt32 Element SAX handler.  
 */
public class DofInt32Element extends DofNumerical
{
    private static String Int32Element = "int32";

    /**
     * DofInt32Element constructor
     * @param childComplete the child completion callback to my parent.  Must not be null.
     */
    public DofInt32Element(ChildComplete childComplete)
    {
        super(childComplete);
    }
    
    public DofInt32Element(int itemId, MetadataElements metadata, Number min, Number max, String unit)
    {
        super(itemId, metadata, min, max, unit);
    }

    @Override
    public void validate(TypedefsElement typedefs) throws SAXException
    {
        super.validate(typedefs);
        String location = "interface.typedefs." + getClass().getSimpleName();
        if(min != null && ((Integer)min) > ((Integer)max))
            throw new SAXException("min > max on a numerical, " + location);
    }

    @Override
    protected int setAttributes(Attributes attributes) throws SAXException
    {
        int length = super.setAttributes(attributes);
        if(length == 1)
            return length;
        if(length < 3)
            throw new SAXException(currentPath() + " if either min or max is given, the other must be given");
         int index = attributes.getIndex(MinAttr);
         if(index < 0)
             throw new SAXException(currentPath() + " expected attribute: " + MinAttr);
         index = attributes.getIndex(MaxAttr);
         this.min = Integer.parseInt(attributes.getValue(index));
         
         index = attributes.getIndex(MaxAttr);
         if(index < 0)
             throw new SAXException(currentPath() + " expected attribute: " + MaxAttr);
         index = attributes.getIndex(MaxAttr);
         this.max = Integer.parseInt(attributes.getValue(index));
         
         if (this.min != null && ((Integer) this.min) > ((Integer) this.max))
             throw new SAXException(currentPath() + " " + getClass().getSimpleName() + " min > max");
         return length;
    }
    
    @Override
    public StringBuilder export(StringBuilder sb, int level)
    {
        return exportNumerical(sb, Int32Element, ""+typeId, min, max, unit, metadata, level);
    }
}
