/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/
package org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.other;

import org.opendof.tools.repository.interfaces.opendof.saxParser.MetadataElements;
import org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.DofTypeBase;
import org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.TypedefsElement;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * DofString Element SAX handler.  
 */
public class DofStringElement extends DofTypeBase
{
    private static String StringElement = "string";
    private static String EncodingAttr = "encoding";

    public volatile int length;
    public volatile int encoding;

    /**
     * DofStringElement constructor
     * @param childComplete the child completion callback to my parent.  Must not be null.
     */
    public DofStringElement(ChildComplete childComplete)
    {
        super(childComplete);
        length = -1;
        encoding = -1;
    }

    public DofStringElement(int itemId, MetadataElements metadata, int length, int encoding)
    {
        super(itemId, metadata);
        this.length = length;
        this.encoding = encoding;
    }
    
    public int getLength()
    {
        return length;
    }
    
    public int getEncoding()
    {
        return encoding;
    }
    
    @Override
    public void validate(TypedefsElement typedefs) throws SAXException
    {
        super.validate(typedefs);
        String location = "interface.typedefs." + getClass().getSimpleName();
        if (length == -1)
            throw new SAXException("Length was not set, " + location);
        if (encoding == -1)
            throw new SAXException("Encoding was not set, " + location);
        if (encoding < 3 || encoding > 32767)
            throw new SAXException("encoding out of range: " + encoding + ", " + location);
    }

    @Override
    protected int setAttributes(Attributes attributes) throws SAXException
    {
        int size = super.setAttributes(attributes);
        if (size < 3)
            throw new SAXException("DofString with less than three attributes");
        if (size > 3)
            throw new SAXException("DofString with more than three attributes");
        int index = attributes.getIndex(LengthAttr);
        if (index < 0)
            throw new SAXException("DofString expected attribute: " + LengthAttr);
        length = Integer.parseInt(attributes.getValue(index));
        index = attributes.getIndex(EncodingAttr);
        if (index < 0)
            throw new SAXException("DofString expected attribute: " + EncodingAttr);
        encoding = Integer.parseInt(attributes.getValue(index));
        if (encoding < 3 || encoding > 32767)
            throw new SAXException("DofString encoding out of range: " + encoding);
        return size;
    }

    @Override
    public StringBuilder export(StringBuilder sb, int level)
    {
        element(sb, StringElement, new String[] { TypeIdAttr, EncodingAttr, LengthAttr }, new String[] { ""+typeId, "" + encoding, "" + length }, null, metadata == null, level);
        return exportNames(sb, StringElement, metadata, true, level);
    }
    
    @Override
    public boolean validPublish(DofTypeBase doftype)
    {
        if(!super.validPublish(doftype))
            return false;
        if(length != ((DofStringElement)doftype).length)
            return false;
        if(encoding != ((DofStringElement)doftype).encoding)
            return false;
        return true;
    }
}
