/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/
package org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.reference;

import org.opendof.tools.repository.interfaces.opendof.saxParser.MetadataElements;
import org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.DofTypeBase;
import org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.TypedefsElement;
import org.opendof.tools.repository.interfaces.opendof.saxParser.typedef.collection.FieldData;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * DofReference Element SAX handler.  
 */
public class DofReference extends DofTypeBase
{
//    public static final String TypeRefElement = "typeref";
//    public static final String TypeRefAttr = "ref";

    protected volatile int reference;

    /**
     * DofReference constructor
     * @param childComplete the child completion callback to my parent.  Must not be null.
     */
    public DofReference(ChildComplete childComplete)
    {
        super(childComplete);
        reference = -1;
    }

    public DofReference(int itemId, int reference, MetadataElements metadata)
    {
        super(itemId, metadata);
        this.reference = reference;
    }
    
    public int getReference()
    {
        return reference;
    }
    
    @Override
    public void validate(TypedefsElement typedefs) throws SAXException
    {
        super.validate(typedefs);
        String location = "interface.typedefs." + getClass().getSimpleName();
        if(reference == -1)
            throw new SAXException("reference was not set, " + location);
        typedefs.validateTyperef(reference);
    }
    
//    public static boolean oneOfYours(String element)
//    {
//        return TypeRefElement.equals(element);
//    }

    @Override
    protected int setAttributes(Attributes attributes) throws SAXException
    {
        int length = super.setAttributes(attributes);
        if(length < 2)
            throw new SAXException(currentPath() + " expected attributes: " + TypeIdAttr + " and " + FieldData.TypeRefAttr);
        int index = attributes.getIndex(TypeIdAttr);
        if (index < 0)
            throw new SAXException(currentPath() + " expected attribute: " + TypeIdAttr);
        typeId = Integer.parseInt(attributes.getValue(index));
        index = attributes.getIndex(FieldData.TypeRefAttr);
        if (index < 0)
            throw new SAXException(currentPath() + " expected attribute: " + FieldData.TypeRefAttr);
        reference = Integer.parseInt(attributes.getValue(index));
        return length;
    }
    
    @Override
    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
    {
        // for references, only metadata is valid as a child
//        if (localName.equals(DofReference.TypeRefElement))
//        {
//            // the stack here will be "interface,typedefs,<type>"
//            stack.push(localName);
//            int size = attributes.getLength();
//            if (size == 0)
//                throw new SAXException(TypeRefElement + " requires  the " + TypeRefAttr + " attribute");
//            if (size > 1)
//                throw new SAXException(TypeRefElement + " has more attributes than expected");
//            int index = attributes.getIndex(TypeRefAttr);
//            reference = Integer.parseInt(attributes.getValue(index));
//            return;
//        }
        if (MetadataElements.oneOfYours(localName))
        {
            // the stack here will be "interface,typedefs,<type>"
            if (metadata == null)
                metadata = new MetadataElements();
            metadata.startElement(uri, localName, qName, attributes);
            return;
        }
        throw new SAXException(currentPath() + "."+localName + " DofType startElement unknown local name for this level");
    }
    
    // NOTE: references have no cdata, so no characters override is needed here
    
    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException
    {
        // the stack here will be "interface,typedefs,<type>,typeref,<metadata>" or "interface,typedefs,<type>,typeref" or "interface,typedefs,<type>"
        // for references, only metadata is valid as a child
//        if (localName.equals(DofReference.TypeRefElement))
//        {
//            stack.pop();
//            return;
//        }
        if (MetadataElements.oneOfYours(localName))
        {
            metadata.endElement(uri, localName, qName);
            return;
        }
        // the stack here will be "interface,typedefs,<type>" let the super clean it up
        super.endElement(uri, localName, qName);
    }
    
    @Override
    public boolean validPublish(DofTypeBase doftype)
    {
        if(!super.validPublish(doftype))
            return false;
        if(reference != ((DofReference)doftype).reference)
            return false;
        return true;
    }
}
