package org.opendof.tools.repository.interfaces.servlet.menu.manage;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

//import org.apache.http.util.ExceptionUtils;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.core.request.handler.IPartialPageRequestHandler;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow.WindowClosedCallback;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RadioChoice;
import org.apache.wicket.markup.html.form.upload.FileUpload;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.util.ListModel;
import org.apache.wicket.protocol.http.WebSession;
import org.apache.wicket.util.lang.Bytes;
import org.apache.wicket.util.value.AttributeMap;
import org.apache.wicket.util.value.IValueMap;
import org.opendof.tools.repository.interfaces.core.AuthenticatedUser;
import org.opendof.tools.repository.interfaces.core.CoreController;
import org.opendof.tools.repository.interfaces.da.DataAccessor;
import org.opendof.tools.repository.interfaces.da.ImmutableException;
import org.opendof.tools.repository.interfaces.da.InterfaceData;
import org.opendof.tools.repository.interfaces.da.SubmitterData;
import org.opendof.tools.repository.interfaces.servlet.InterfaceRepository;
import org.opendof.tools.repository.interfaces.servlet.auth.ClientInformation;
import org.opendof.tools.repository.interfaces.servlet.definition.Definition;
import org.opendof.tools.repository.interfaces.servlet.index.Index;
import org.opendof.tools.repository.interfaces.servlet.menu.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;

public class ManageInterfacePanel extends InterfacePanel {
    private static final long serialVersionUID = -4736200814044302198L;

    private static final String publicOption = "Public";
    private static final String privateOption = "Private";

    private final InterfaceData interfaceData;
    private ModalWindow publishWindow;
    private ConfirmPanel publishConfirmPanel;
    private ModalWindow accessWindow;
    private AccessConfirmPanel accessConfirmPanel;
    private ModalWindow submitWindow;
    private ConfirmPanel submitConfirmPanel;
    private FileUpload uploadedFile = null;
    private ModalWindow deleteWindow;
    private AccessRadioChoice accessChoice;
    private AuthenticatedUser authUser = null;
    private final ModalWindow notificationWindow;
    private final NotificationPanel notificationPanel;

    private boolean currentPublish;
    private String currentAccess;

    private IModel<String> publishedLabelModel;
    private ConfirmationLink publishButton;
    private ConfirmationLink visibilityButton;
    private ConfirmationLink deleteButton;

    private FileUploadField fileUploadField;

    private static final Logger log = LoggerFactory.getLogger(ManageInterfacePanel.class);

    public ManageInterfacePanel(String id, InterfaceData interfaceData, boolean owner, boolean hasWrite) {
        super(id);
        this.interfaceData = interfaceData;
        this.currentAccess = Objects.equals(interfaceData.accessGroup.email, DataAccessor.PrivateGroupEmail) ? privateOption : publicOption;
        this.currentPublish = interfaceData.publish;

        addPublishOptions();
        addAccessOptions();
        addSubmitOptions(hasWrite);
        addDeleteOptions(owner);
        accessChoice.setVisible(owner);
        publishButton.setVisible(owner);
        visibilityButton.setVisible(owner);

        notificationWindow = new ConfirmationModalWindow("notificationWindow", "Error", false);
        notificationPanel = new NotificationPanel(notificationWindow, "", "");
        add(notificationWindow);


        if (getClientInformation().isAuthenticated())
            authUser = new AuthenticatedUser(getClientInformation().getName(), getClientInformation().getEmail(), getClientInformation().getDescription(), null);
    }

    private void addAccessOptions() {
        IModel<List<String>> accessChoices = new ListModel<>(Arrays.asList(privateOption, publicOption));
        accessChoice = new AccessRadioChoice("accessChoice", accessChoices);
        add(accessChoice);
        accessChoice.setEnabled(!(Objects.equals(currentAccess, publicOption) && currentPublish));

        accessWindow = new ConfirmationModalWindow("confirmAccess", "Access Change", true);

        visibilityButton = new ConfirmationLink("visibilityButton", accessWindow);
        add(visibilityButton);

        visibilityButton.setVisible(!(currentPublish && Objects.equals(currentAccess, publicOption)));

        AccessConfirmationHandler accessConfirmationHandler = new AccessConfirmationHandler();
        accessConfirmPanel = new AccessConfirmPanel(accessWindow, accessConfirmationHandler, "", "", "OK", "Close", currentAccess);
        accessWindow.setWindowClosedCallback(accessConfirmationHandler);

        add(accessWindow);
    }

    private void addSubmitOptions(boolean hasWrite) {

        submitWindow = new ConfirmationModalWindow("confirmSubmit", "Submit InterfaceDefinition", false);
        SubmitConfirmationHandler submitConfirmationHandler = new SubmitConfirmationHandler();
        submitConfirmPanel = new ConfirmPanel(submitWindow, submitConfirmationHandler, "", "", "Submit", "Cancel");
        submitWindow.setWindowClosedCallback(submitConfirmationHandler);

        Form<Void> submitForm = new SubmitForm("submitForm");

        submitForm.setMultiPart(true);
        submitForm.setMaxSize(Bytes.megabytes(2));

        fileUploadField = new FileUploadField("upload");
        submitForm.add(fileUploadField);
        FormConfirmationLink submitButton = new FormConfirmationLink("submitButton", submitWindow, submitForm);
        submitForm.add(submitButton);
        add(submitForm);
        submitForm.setVisible(hasWrite);
        visibilityButton.setVisible(!(currentPublish && Objects.equals(currentAccess, publicOption)));

        add(submitWindow);
    }

    private void addPublishOptions() {
        publishWindow = new ConfirmationModalWindow("confirmPublish", "Publish Interface", true);

        PublishConfirmationHandler publishConfirmationHandler = new PublishConfirmationHandler();
        publishWindow.setWindowClosedCallback(publishConfirmationHandler);
        publishConfirmPanel = new ConfirmPanel(publishWindow, publishConfirmationHandler, null, null, "Publish", "Cancel");
        setPublishWindowMessages();

        String publishedMessage = "This interface is " + (currentPublish ? "published." : "working: ");

        publishedLabelModel = new Model<>(publishedMessage);
        Label publishedLabel = new Label("publishedLabel", publishedLabelModel);
        publishedLabel.setOutputMarkupId(true);
        add(publishedLabel);
        publishButton = new ConfirmationLink("publishButton", publishWindow);
        publishButton.setVisible(!currentPublish && interfaceData.xml != null);

        add(publishButton);
        add(publishWindow);
    }

    private void addDeleteOptions(boolean owner) {
        deleteWindow = new ConfirmationModalWindow("confirmDelete", "Delete Interface", false);
        DeleteConfirmationHandler deleteConfirmationHandler = new DeleteConfirmationHandler();
        deleteWindow.setWindowClosedCallback(deleteConfirmationHandler);

        String msg1 = "Are you sure you want to delete " + interfaceData.iid + '?';
        String msg2 = "After deleting an interface you will no longer have access to the interface identifier and must allocate another.";
        ConfirmPanel deleteConfirmPanel = new ConfirmPanel(deleteWindow, deleteConfirmationHandler, msg1, msg2, "Delete", "Cancel");

        deleteButton = new ConfirmationLink("deleteButton", deleteWindow);
        deleteButton.setVisible(!currentPublish && owner);

        add(deleteButton);
        add(deleteWindow);
    }

    private void setPublishWindowMessages() {
        String confirmMsg1 = "Are you sure you want to publish " + interfaceData.iid + '?';
        String confirmMsg2 = "Published interfaces cannot be deleted. Changes to published interface definitions " +
                "are restricted to documentation, so ensure that you have completed your organization's " +
                "review process before publishing.";

        if (Objects.equals(currentAccess, publicOption)) {
            confirmMsg2 += " This is a public interface. Public interfaces may not be made private once published.";
        }
        publishConfirmPanel.setMessages(confirmMsg1, confirmMsg2);
    }

    private final class SubmitForm extends Form<Void> {

        private SubmitForm(String id) {
            super(id);
        }

        private static final long serialVersionUID = -4431707972578398057L;

        @Override
        protected void onSubmit() {
            uploadedFile = fileUploadField.getFileUpload();

            String msg1;
            String msg2;
            if (uploadedFile != null) {
                msg1 = "Are you sure you want to update the interface definition to the contents of " + uploadedFile.getClientFileName() + '?';
                msg2 = "No change history is kept. You will not be able to revert this change.";
                submitConfirmPanel.enableConfirmButton();
                submitConfirmPanel.setCancelButtonText("Cancel");
            } else {
                msg1 = "No file selected.";
                msg2 = "";
                submitConfirmPanel.disableConfirmButton();
                submitConfirmPanel.setCancelButtonText("Close");
            }
            submitConfirmPanel.setMessages(msg1, msg2);
        }
    }

    private class PublishConfirmationHandler implements ConfirmationHandler, WindowClosedCallback {
        private static final long serialVersionUID = 2316940336271416989L;

        private boolean isError = false;
        private String errorMessage = null;

        @Override
        public void onConfirm(AjaxRequestTarget target) {
            try {
                getCore().publishInterface(authUser, interfaceData.getRepoType(), interfaceData.iid, interfaceData.version);
                log.debug("Published interface " + interfaceData.iid);
                onInterfaceUpdate(target, true, currentAccess);
            } catch (Exception e) {
                log.debug("Failed to publish interface " + interfaceData.iid + " - " + e, e);
                isError = true;
                errorMessage = e.toString();
            } finally {
                publishWindow.close(target);
            }
        }

        @Override
        public void onCancel(AjaxRequestTarget target) {
            publishWindow.close(target);
        }

        @Override
        public void onClose(AjaxRequestTarget target) {
            if (isError) {
                notificationPanel.setMessages("Error", "Failed to publish interface.", errorMessage);
                notificationWindow.show(target);
            }
            isError = false;
        }
    }

    private class SubmitConfirmationHandler implements ConfirmationHandler, WindowClosedCallback {
        private static final long serialVersionUID = 8490632423427084563L;

        private boolean isError = false;
        private String errorMessage = null;

        @Override
        public void onConfirm(AjaxRequestTarget target) {
            try {
                if (uploadedFile != null) {
                    String xml = new String(uploadedFile.getBytes());
                    getCore().updateInterface(authUser, interfaceData.getRepoType(), interfaceData.iid, xml, interfaceData.version, null, null, null);
                }
                log.debug("Submitted interface definiton for " + interfaceData.iid);
                onInterfaceUpdate(target, currentPublish, currentAccess);
                setResponsePage(Definition.class, getPage().getPageParameters());
            } catch (ImmutableException ignored) {
                isError = true;
                errorMessage = "Attempting to modify an immutable element/attribute of a published interface. " +
                        "Only metadata may be modified on published interfaces.";
            } catch (SAXException se) {
                isError = true;
                log.debug("Failed to submit interface definition for " + interfaceData.iid + " - " + se, se);
                errorMessage = "Invalid Interface Definition:\n" + se.getMessage();
            } catch (Exception e) {
                isError = true;
                log.warn("Failed to submit interface definition for " + interfaceData.iid + " - " + e, e);
                StringWriter sw = new StringWriter();
                e.printStackTrace(new PrintWriter(sw));
                String stackTrace = sw.toString();
                errorMessage = "An unexpected error occurred. Please contact the system administrator. \n" + stackTrace;
            } finally {
                submitWindow.close(target);
            }
        }

        @Override
        public void onCancel(AjaxRequestTarget target) {
            submitWindow.close(target);
        }

        @Override
        public void onClose(AjaxRequestTarget target) {
            if (isError) {
                notificationPanel.setMessages("Error", "Failed to update interface definition.", errorMessage);
                notificationWindow.show(target);
            }
            isError = false;
        }
    }

    private void onInterfaceUpdate(IPartialPageRequestHandler target, boolean publish, String visibility) {
        currentPublish = publish;
        currentAccess = visibility;

        //Publish Options
        if (currentPublish) {
            publishedLabelModel.setObject("This interface is published.");
        }
        publishButton.setVisible(!currentPublish && interfaceData.xml != null);
        target.add(publishButton);
        setPublishWindowMessages();
        accessConfirmPanel.updateMessage();
        //Visibility options
        if(accessChoice.isVisible()) {
            accessChoice.setEnabled(!(Objects.equals(currentAccess, publicOption) && currentPublish));
            target.add(accessChoice);
        }
        visibilityButton.setVisible(!(currentPublish && Objects.equals(currentAccess, publicOption)));
        target.add(visibilityButton);

        //Delete Options
        deleteButton.setVisible(!currentPublish);
        target.add(deleteButton);
    }

    private final class AccessConfirmPanel extends ConfirmPanel {
        private static final long serialVersionUID = 840919396153656322L;

        private String selectedAccess = null;

        private AccessConfirmPanel(ModalWindow window, ConfirmationHandler confirmationHandler, String confirmationMsg1,
                                   String confirmationMsg2, String confirmText, String cancelText, String selectedAccess) {
            super(window, confirmationHandler, confirmationMsg1, confirmationMsg2, confirmText, cancelText);

            setAccess(selectedAccess);
        }

        public void updateMessage() {
            String msg1;
            String msg2;
            if (Objects.equals(currentAccess, selectedAccess)) {
                msg1 = "Interface is already set to " + selectedAccess;
                msg2 = "";
                disableConfirmButton();
                setCancelButtonText("Close");
            } else if (Objects.equals(selectedAccess, publicOption)) {
                msg1 = "Interface will be set to public.";
                msg2 = "Public interfaces may be viewable by anyone.";
                if (currentPublish) {
                    msg2 += " This interface is published. Once made public, published interfaces cannot be made private again.";
                }
                enableConfirmButton();
                setCancelButtonText("Cancel");
            } else {
                msg1 = "Interface will be set to private.";
                msg2 = "Private interfaces may only be viewed by you.";
                enableConfirmButton();
                setCancelButtonText("Cancel");
            }
            setMessages(msg1, msg2);
        }

        private void setAccess(String access) {
            selectedAccess = access;
            updateMessage();
        }
        private String getSelectedAccess() {
            return selectedAccess;
        }
    }

    private final class AccessRadioChoice extends RadioChoice<String> {
        private static final long serialVersionUID = -498306121179292730L;

        private AccessRadioChoice(String id, IModel<? extends List<? extends String>> choices) {
            super(id, new Model<>(currentAccess), choices);
        }

        @Override
        protected boolean wantOnSelectionChangedNotifications() {
            return true;
        }

        @Override
        protected void onSelectionChanged(String newSelection) {
            accessConfirmPanel.setAccess(newSelection);
        }

        @Override
        protected IValueMap getAdditionalAttributesForLabel(int index, String choice) {
            if (Objects.equals(choice, currentAccess)) {
                IValueMap attributeMap = new AttributeMap();
                attributeMap.put("style", "font-style: italic;");
                return attributeMap;
            }
            return null;
        }
    }

    private class AccessConfirmationHandler implements ConfirmationHandler, WindowClosedCallback {
        private static final long serialVersionUID = 6718630523755175586L;

        private boolean isError = false;
        private String errorMessage = null;

        @Override
        public void onConfirm(AjaxRequestTarget target) {
            String selectedAccess = accessConfirmPanel.getSelectedAccess();
            try {
                SubmitterData accessGroup = selectedAccess.equals(privateOption) ? DataAccessor.PrivateGroup : DataAccessor.AnonymousGroup;
                getCore().updateInterface(authUser, interfaceData.getRepoType(), interfaceData.iid, interfaceData.xml, interfaceData.version, interfaceData.submitter, accessGroup, null);

                onInterfaceUpdate(target, currentPublish, selectedAccess);
                log.debug("Set interface " + interfaceData.iid + " to " + selectedAccess);
            } catch (Exception e) {
                isError = true;
                log.debug("Failed to set interface " + interfaceData.iid + " to " + selectedAccess + " - " + e, e);
                errorMessage = e.toString();
            } finally {
                accessWindow.close(target);
            }
        }

        @Override
        public void onCancel(AjaxRequestTarget target) {
            accessWindow.close(target);
        }

        @Override
        public void onClose(AjaxRequestTarget target) {
            if (isError) {
                notificationPanel.setMessages("Error", "Failed to update interface visibility.", errorMessage);
                notificationWindow.show(target);
            }
            isError = false;
        }
    }

    private class DeleteConfirmationHandler implements ConfirmationHandler, WindowClosedCallback {
        private static final long serialVersionUID = -5102019936712886844L;

        private boolean isError = false;
        private String errorMessage = null;

        @Override
        public void onConfirm(AjaxRequestTarget target) {
            try {
                getCore().deleteInterface(authUser, interfaceData.getRepoType(), interfaceData.iid, interfaceData.version);
                log.debug("Deleted interface " + interfaceData.iid);
                setResponsePage(Index.class);
            } catch (Exception e) {
                isError = true;
                log.debug("Failed to delete interface " + interfaceData.iid + " - " + e, e);
                errorMessage = e.toString();
            } finally {
                deleteWindow.close(target);
            }
        }

        @Override
        public void onCancel(AjaxRequestTarget target) {
            deleteWindow.close(target);
        }

        @Override
        public void onClose(AjaxRequestTarget target) {
            if (isError) {
                notificationPanel.setMessages("Error", "Failed to delete interface.", errorMessage);
                notificationWindow.show(target);
            }
            isError = false;
        }
    }
}
