package org.opendof.tools.repository.interfaces.servlet.menu.sharing;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormValidatingBehavior;
import org.apache.wicket.feedback.ExactLevelFeedbackMessageFilter;
import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.feedback.IFeedbackMessageFilter;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.time.Duration;
import org.apache.wicket.validation.validator.EmailAddressValidator;
import org.opendof.tools.repository.interfaces.core.AuthenticatedUser;
import org.opendof.tools.repository.interfaces.da.InterfaceData;
import org.opendof.tools.repository.interfaces.da.SubmitterData;
import org.opendof.tools.repository.interfaces.da.SubmitterPermissions;
import org.opendof.tools.repository.interfaces.servlet.menu.InterfacePanel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.apache.wicket.markup.html.list.ListView;

import java.io.Serializable;
import java.util.*;

/**
 * Created by dethington on 7/17/2017.
 */
public class ManageSharingPanel extends InterfacePanel {

    private static final long serialVersionUID = -3210429451291738008L;
    private final InterfaceData interfaceData;
    private final AuthenticatedUser authUser;
    private static final Logger log = LoggerFactory.getLogger(ManageSharingPanel.class);
    private ListView<PermissionsWrapper> listView = null;

    private Boolean fakeData = Boolean.FALSE;
    private static final List<String> fakeEmails;
    private static final Map<String, PermissionsWrapper> fakePermissons;
    private final FeedbackPanel feedbackPanel = new EmailErrorFeedbackPanel("feedbackPanel", new ExactLevelFeedbackMessageFilter(FeedbackMessage.ERROR));

    static {
        fakeEmails = new ArrayList<>();
        fakeEmails.add("fakeuser1@example.com");
        fakeEmails.add("fakeuser2@example.com");
        fakeEmails.add("fakeuser3@example.com");
        fakeEmails.add("fakeuser4@example.com");
        fakePermissons = new HashMap<>();
        int x = 0;
        for(String email : fakeEmails){
            Boolean writable = x % 2 > 0;
            SubmitterPermissions.PermissionLevel perm = SubmitterPermissions.PermissionLevel.READ;
            if(writable)
                perm = SubmitterPermissions.PermissionLevel.WRITE;
            SubmitterData sub = new SubmitterData("Name of " + email,
                    email, "Description of " + email,
                    new Date(), false );
            SubmitterPermissions perm1 = new SubmitterPermissions(sub, perm);
            fakePermissons.put(email, new PermissionsWrapper(perm1));
            x++;
        }

    }

    public ManageSharingPanel(String id, InterfaceData interfaceData) {
        super(id);
        this.interfaceData = interfaceData;
        authUser = getClientInformation().isAuthenticated() ? getAuthUser() : null;
        loadSubmitterPermissionList();
        add(new AddSubmitterForm("newsubmitter"));
        add(listView);
    }

    private void addNewRow(String email, SubmitterPermissions.PermissionLevel permission) throws Exception {
        if(fakeData){
            SubmitterData sub = new SubmitterData("Name of " + email,
                    email, "Description of " + email,
                    new Date(), false );
            SubmitterPermissions perm1 = new SubmitterPermissions(sub, permission);
            fakePermissons.put(email, new PermissionsWrapper(perm1));
        }else {
                getCore().setPermissions(authUser, interfaceData.getRepoType(), interfaceData.iid, interfaceData.version,
                        email, permission.getValue());
        }
        loadSubmitterPermissionList();
    }

    private void updateListItem(PermissionsWrapper permissionsWrapper) {
        if(fakeData && fakePermissons.containsKey(permissionsWrapper.email)){
            PermissionsWrapper target = fakePermissons.get(permissionsWrapper.email);
            target.setSelected(permissionsWrapper.getSelected());
        }else {
            try {
                SubmitterPermissions.PermissionLevel permissionLevel = SubmitterPermissions.PermissionLevel.READ;
                if (permissionsWrapper.selected)
                    permissionLevel = SubmitterPermissions.PermissionLevel.WRITE;
                getCore().setPermissions(authUser, interfaceData.getRepoType(), interfaceData.iid,
                        interfaceData.version, permissionsWrapper.email, permissionLevel.getValue());
            } catch (Exception e) {
                log.debug("Exception while attempting to update item. ", e);
            }
        }
        loadSubmitterPermissionList();
    }

    private void deleteListItem(PermissionsWrapper permissionsWrapper) {
        if(fakeData  && fakePermissons.containsKey(permissionsWrapper.email)) {
            fakePermissons.remove(permissionsWrapper.email);
        }else {
            try {
                getCore().revokePermissions(authUser, interfaceData.getRepoType(), interfaceData.iid,
                        interfaceData.version, permissionsWrapper.email);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        loadSubmitterPermissionList();
    }


    private void loadSubmitterPermissionList(){
        List<PermissionsWrapper> permissionsList = new ArrayList<>();
        if(authUser != null){
            try {
                List<SubmitterPermissions> submitterPermissionsList = getCore().getPermissions(authUser, interfaceData.getRepoType(), interfaceData.iid, interfaceData.version);
                for(SubmitterPermissions permissions : submitterPermissionsList){
                    permissionsList.add(new PermissionsWrapper(permissions));
                }
            } catch(Exception e){
                log.debug("Failed to get permissions.  Exception: {}", e.getMessage(), e);
                e.printStackTrace();
            }
        }
        if(fakeData){
           for(Map.Entry<String, PermissionsWrapper> stringPermissionsWrapperEntry : fakePermissons.entrySet()){
               permissionsList.add(stringPermissionsWrapperEntry.getValue());
           }
        }
        if(listView == null)
            listView = new PermissionsWrapperListView(permissionsList);
        else
            listView.setModelObject(permissionsList);
    }

    private static class PermissionsWrapper implements Serializable {
        private static final long serialVersionUID = -6072944005630711339L;

        private final String email;
        private Boolean selected = Boolean.FALSE;

        PermissionsWrapper(SubmitterPermissions permissions){
            email = permissions.submitter.email;
            selected = Objects.equals(permissions.permission, SubmitterPermissions.PermissionLevel.WRITE);
        }

        Boolean getSelected() {
            return selected;
        }

        public void setSelected(Boolean selected) {
            this.selected = selected;
        }

        public String getEmail() {
            return email;
        }
    }

    private class PermissionsWrapperListView extends ListView<PermissionsWrapper> {
        private static final long serialVersionUID = 5374952604176542286L;

        public PermissionsWrapperListView(List<PermissionsWrapper> permissionsList) {
            super("listView", permissionsList);
        }

        @Override
        protected void populateItem(ListItem<PermissionsWrapper> item){
            PermissionsWrapper submitterPermission = item.getModelObject();
            item.add(new ChangePermissionForm("itemForm",  submitterPermission));
        }
    }

    private class PermissionCheckBox extends CheckBox {

        private static final long serialVersionUID = -2571870080511164511L;

        private PermissionsWrapper permissionsWrapper;

        PermissionCheckBox(String id, IModel<Boolean> model, PermissionsWrapper item) {
            super(id, model);
            permissionsWrapper = item;
        }

        @Override
        protected boolean wantOnSelectionChangedNotifications(){
            return true;
        }

        @Override
        protected void onModelChanged() {
            permissionsWrapper.selected = getModelObject();
            updateListItem(permissionsWrapper);
        }
    }

    private class ChangePermissionForm extends Form {

        private static final long serialVersionUID = -8364348478612416482L;

        final PermissionsWrapper permissionsWrapper;

        ChangePermissionForm(String id, PermissionsWrapper item) {
            super(id);
            this.permissionsWrapper = item;
            add(new Label("email", item.getEmail()));
            add(new PermissionCheckBox("cb", Model.of(item.getSelected()), item));
            add(new Button("delete"));
        }

        @Override
        protected void onSubmit() {
            deleteListItem(permissionsWrapper);
        }
    }

    private class AddSubmitterForm extends Form {
        private static final long serialVersionUID = 5279764626091045163L;
        private TextField<String> emailField;
        private CheckBox canWrite;
        private Button submit;

        AddSubmitterForm(String id){
            super(id);

            emailField = new TextField<>("emailField", Model.of(""));
            emailField.add(EmailAddressValidator.getInstance());
            add(new AjaxFormValidatingBehavior("submit", Duration.ONE_SECOND){
                @Override
                protected void onError(AjaxRequestTarget target) {
                    add(feedbackPanel);
                }
            });
            canWrite = new CheckBox("canWrite", Model.of(Boolean.FALSE) );
            submit = new Button("submit");
            add(emailField);
            add(canWrite);
            add(submit);
            add(feedbackPanel);
        }

        @Override
        public final void onSubmit(){
            String email = emailField.getDefaultModelObjectAsString();
            if(Objects.equals(email, authUser.email)){
                error("Owner email, " + email + ", automatically has full access.");
                return;
            }
            Boolean hasWrite = canWrite.getModelObject();
            SubmitterPermissions.PermissionLevel permissionLevel = SubmitterPermissions.PermissionLevel.READ;
            if(hasWrite)
                permissionLevel = SubmitterPermissions.PermissionLevel.WRITE;
            try {
                addNewRow(email, permissionLevel);
                emailField.setModelObject("");
                canWrite.setModelObject(Boolean.FALSE);
            } catch (Exception e) {
                log.debug("Exception: ", e);
                error("Submitter not found: " + email);
            }
        }
    }

    private static final class EmailErrorFeedbackPanel extends FeedbackPanel{

        private static final long serialVersionUID = -8676848049778931167L;

        private EmailErrorFeedbackPanel(String id, IFeedbackMessageFilter filter) {
            super(id, filter);
        }

        @Override
        public Component newMessageDisplayComponent(String id, FeedbackMessage message){
            if(message.getMessage().toString().contains("not found") || message.getMessage().toString().contains("full access"))
                return new Label(id, message.getMessage().toString());
            return new Label(id, "Incorrect email format");
        }
    }
}
