Interface Repository
====
***

The Interface Repository is a registry of interface definitions for IoT(Internet of Things) technologies. 

A primary goal of the Interface Repository is to foster interoperability between devices with similar functionality. Published interfaces define a
fixed set of common functionality and allow devices from many different providers to communicate as long as they are written against the same
interfaces.


## Interface Definitions
***

The raw format of an interface definition is generally defined by the organization which sponsors the technology used by that interface. OpenDOF,
for example, defines an interface definition as an XML document which validates against the XML schema found at 
http://www.opendof.org/schema/interface-repository. Interface definitions may also be translated by the Interface Repository into other formats.

#### Versioning

The Interface Repository supports versioning of interfaces. Different IoT technologies may have different rules for interface versioning and
may not allow it at all.

#### Human Readable

The Interface Repository displays a human readable HTML translation of the raw interface definitions by default. Localization
of metadata is supported.

#### Machine Readable

In addition to the raw interface definition format, the Interface Repository also provides the ability to generate code representing
the interface definition in whatever programming languages are supported by the IoT technology used by the interface.

#### Reserve Interface Identifiers

Interface definitions are identified by interface identifiers which must be unique to a particular communication protocol. For example, each OpenDOF interface must have an identifier unique among the set of all OpenDOF interfaces. However, an AllJoyn interface may share the same identifier as an OpenDOF interface.

The Interface Repository tracks which interfaces are being used or have been reserved. Authenticated users may use the website to reserve interface identifiers which will be associated with their account.

Each communication protocol may have different rules for assigning interface identifiers.

#### Upload Interface Definitions

Authenticated users may upload an interface definition for interface identifiers associated with their account.

#### Searching

The Interface Repository website includes a search box allowing users to search for individual interfaces. The website must be indexable by major search engines such as Google.


## Managing the Interface Repository
***

The Interface Repository provides a command-line tool for managing interface definitions. More information about the command-line tool can
be found [here](../interface-repository-cli/README.md) 
	
## Browse the Interface Repository
***

There are several ways to find existing interface definitions in the Interface Repository.

If the interface identifier is known, users may go directly to the interface definition from the home page.

If the website is configured to allow search engine indexing, interface definitions may be found via Google or other search engine.

The Interface Repository also has a browse page where users may view all known interface definitions. Interfaces may be filtered by
repository type, submitter or by publish state.

## Web API
***

Applications may use the Web API to access the raw format of an interface definition using an HTTP GET on the URL 
https://<www.someinterfacerepository.com>/interface/raw/<repo>/<iid>/<version>?trans=XML with the following parameters:
	
	* iid - Required. Specifies the identifier of the interface to be returned.
	* repo - Required. Specify the repository type associated with the interface definition. For example, opendof.
	* version - Optional. The version of the interface definition to return. Defaults to 1 if not specified. Do not specify if versions are not supported for the specified repo.

Example:

    https://interface.opendof.org/interface/raw/opendof/1.01/1?trans=XML
	
## Install Instructions
Installation instructions for the Interface Repository can be found [here](./doc/INSTALL.md).

## Build Instructions
This project uses [Apache Ant](http://ant.apache.org/) and [Apache Ivy](http://ant.apache.org/ivy/) to build and resolve dependencies. In addition, 
a Java Development Kit (JDK) of 1.7 or greater is required.

To build, make sure any dependencies are available in a configured Ivy repository, and execute 'ant dist' in the root of the project (this is the 
default ant target). To publish the build to your local ivy repository (and make it available for other builds to depend on), execute 'ant publish'.

If you plan to make a complete distribution including source, you will also need to use the 'dist-src' and 'publish-src' targets.
Typically, these would be executed prior to the binary build ('dist' and 'publish' targets), since they require a clean directory.

## Runtime Prerequisites
A Java Runtime Environment (JRE) of 1.8 or greater is required.

## How to Report an Issue
Issues can be reported through the issue tracking system of the OpenDOF Project at https://issue.opendof.org

Information on the process for reporting an issue can be found at https://opendof.org/report-an-issue.
