Interface Repository Test Plan
====
Date 08-24-2017

# Introduction
***

This document lays out the proposed test strategies for the Interface Repository.  All sections reflect the current requirements and functionality available from version 1.0/2.0 of the Interface Repository. Any new requirements and/or functionality that is only available in version 2.0 of the Interface Repository will be labeled with "since v2.0".

#Test Strategy
***

## Functionality Testing

- Installation: Proper installation of the Apache Tomcat Interface Repository servlet and irmanage commandline management tool will be performed, following the available documentation for version 2.0.

- Migration: Verification of successful migration from version 1.x to 2.0 will be performed, using a previous subset of 1.x data from a MySQL database.  This will then be migrated to version 2.0 using the provided documentation steps.

- Interface Repository GUI (Interface Definition): Verify versioning of interfaces are possible when allowed, and that they are shown on the GUI. Verify that raw interface definitions (XML) can be displayed via (HTML) on the site in a human readable translation. Verify that raw interface definitions can be converted to, or that code can be generated in whatever languages supported by the IoT technology used by the interface.

- Interface Repository GUI (Authentication): All functionality listed here is since version 2.0. Verify that a user can authenticate using OAuth technology.  Verify that an Authenticated User can reserve interface identifiers using the Iterface Repository site, which will be verified second fold by viewing the association in the database under the user account. Verify that an Authenticated User can upload an interface definition for interface identifiers associated with their account.

- Interface Repository GUI (Searching/Browsing): Verify that a user, whether authenticated or not, can search for individual interfaces using the "Search Box".  In order, to test this the Interface Repository configuration will be setup to be indexable by major search engines. Verify that a user can select a known interface identifier directly from the home page to view it.  Verify that a user can search for an interface definition by using Google, in order to test this, the configuration of the site will be setup to be indexable. Verify that a user may filter and view all known interface definitions by repository type, submitter, or by publish state.

- Interface Repository Management: Verify that the Interface Repository can be managed via the provided command-line tool. Verify that one can add, list, update, delete, and Sync (since version 2.0) interfaces. Verify that one can add, list, update, delete submitters.  Verify that one can add, list, update, delete groups.  Verify that one can add, list, update, delete sub-repositories (since version 2.0).

- Web API Accessiblity: Verify that an application can access the raw format of an interface definition using the provided Web API.  Specifically execute an HTTP GET on the respectful URL, following the guidelines in the Interface Repository README.

- Shareable Access to Private Interfaces: Verify that a user who owns a submitted interface can share access to the particular interface either through the CLI tool or GUI (since version 2.1).

## Integration Testing

- OpenDOF: Where the Interface Repository is designed to provide any number of controllers for interface definitions from various IoT Technologies, it is imperative to validate the current ones supplied out-of-the box. One of the supplied controllers is for OpenDOF Interfaces.  Using the GUI/CLI tool, interface definitions from OpenDOF will be verified for correct reservation, association, viewing, and updating/deleting. Also, where OpenDOF provides the DOFConsole that is able to import and use OpenDOF interfaces for DOF Objects, the DOFConsole for all current versions will be tested against version 2.0 of the Interface Repository.

- AllJoyn: Where the Interface Repository is designed to provide any number of controllers for interface definitions from various IoT Technologies, it is imperative to validate the current ones supplied out-of-the box. One of the supplied controllers is for AllJoyn Interfaces.  Using the GUI/CLI tool, interface definitions from AllJoyn will be verified for correct reservation, association, viewing, and updating/deleting.

## Negative Testing

- GUI: Verify that authentication will fail with incorrect credentials. Verify that Reservations will fail if not authenticated, and if the desire interface identifier is already in use by the respective IoT technology. Verify that an interface will fail to be uploaded if it already exists or doesn't validate against the interface definition XML schema. Do this for OpenDOF and AllJoyn interfaces.

- CLI: Verify that an interface fails to be added, updated if the interface doesn't validate against the interface definition XML schema.  Do this for OpenDOF and AllJoyn.  Verify that adding, updating, and deleting fails for interfaces, groups, submitters, and sub-repositories when provided parameters are incorrect.

## Longterm Testing

Where it was deemed unnecessary to stress test high amounts of load on the Interface Repository web application, the degree of long-term testing is also minimal. The most concern will be to verify CPU usage is minimal and no memory leaks happen on the instance running the Interface Repository web application.

In order to substantiate the reliablitiy of the Interface Repository, the web application will be ran idle and under small load over periods of days monitoring the CPU usage and memory of the instance.  Small load is defined as database manipulation via the CLI tool and any browsing or database manipulation available on the site itself.


## Scalabitility Testing

N/A.

## Performance Testing

Though the Interface Repository is a web application, the amount of interaction/load that can take place on the actual server was deemed unnecessary to test for the current use of the product.  This may change in the future, where-in load tests will be performed.

# Test Items
***
The following breaks down the amount of test coverage of the Interface Repository from both Engineering and Quality Assurance. It is primarily listed here to give a high level understanding for interested parties on how the Interface Repository is qualified.

 * System Level Tests - 61 tests